#ifndef SAA713X_REG_H
#define SAA713X_REG_H

#define SAA_VENDOR_ID		0x1131
#define SAA7130_DEV_ID		0x7130
#define SAA7133_DEV_ID		0x7133
#define SAA7134_DEV_ID		0x7134
#define SAA7135_DEV_ID		0x7135

#define SAA_DMA_NRCHANS		0x07
#define SAA_FIFO_NRSLICES	0x04
#define SAA_I2C_ADDR_EEPROM	0xa0
#define SAA_PCIR_MEMBASE	0x10

#define SAA_CHAN_TASKA		0
#define SAA_CHAN_TASKB		1
#define SAA_CHAN_VBIA		2
#define SAA_CHAN_VBIB		3
#define SAA_CHAN_AUDIO		6
#define SAA_CHAN_YUV_Y		0
#define SAA_CHAN_YUV_U		4
#define SAA_CHAN_YUV_V		5

#define SAA_MAX_CLIPS		8


/* DMA and control Registers */
#define REG_DMA_BASE0(chan)	(0x200 + (0x10 * chan))
#define REG_DMA_BASE1(chan)	(0x204 + (0x10 * chan))
#define REG_DMA_PITCH(chan)	(0x208 + (0x10 * chan))

#define REG_DMA_CTRL(chan)	(0x20c + (0x10 * chan))
#define 	DMA_CTRL_SWAPBYTES_SHIFT	24
#define 	DMA_CTRL_SWAPBYTES_MASK		(0x3 << 24)
		enum SWAPBYTES {
			SWAPBYTES_NONE = 0x0 << 24,
			SWAPBYTES_2143 = 0x1 << 24,
			SWAPBYTES_3142 = 0x2 << 24,
			SWAPBYTES_4321 = 0x3 << 24,
		};
#define 	DMA_CTRL_BURST_SHIFT		21
#define 	DMA_CTRL_BURST_MASK		(0x7 << 21)
#define 	DMA_CTRL_ME			(1 << 20)
#define		DMA_CTRL_PTA_SHIFT		0
#define 	DMA_CTRL_PTA_MASK		(0xfffff << 0)

#define REG_FIFO_CFG	0x2A0
#define 	F_C(n)			(((n) / 128) % 9)
#define 	FIFO_SET_SIZE(a, b, c, d) REG_WRITE(REG_FIFO_CFG, \
				 		(F_C(a) | \
				 		(F_C(a + b) << 8) | \
				 		(F_C(a + b + c) << 16) | \
				 		(0x08 << 24)) )

#define REG_FIFO_THRESH	0x2A4
#define		FIFO_SET_THRESH(a, b, c, d) REG_WRITE(REG_FIFO_THRESH, \
						((a % 4) | \
				 		((b % 4) << 8) | \
				 		((c % 4) << 16) | \
				 		((d % 4) << 24)) )

#define REG_MCR		0x2A8
#define 	MCR_VPLLE		(1 << 15)
#define		MCR_APLLE		(1 << 14)
#define 	MCR_EXOSC		(1 << 13)
#define 	MCR_EVFE1		(1 << 12)
#define		MCR_EVFE2		(1 << 11)
#define		MCR_ESFE		(1 << 10)
#define		MCR_EBADC		(1 << 9)
#define		MCR_EBDAC		(1 << 8)
#define		MCR_CHAN6_TE		(1 << 6)
#define		MCR_CHAN5_TE		(1 << 5)
#define		MCR_CHAN4_TE		(1 << 4)
#define		MCR_CHAN3_TE		(1 << 3)
#define		MCR_CHAN2_TE		(1 << 2)
#define		MCR_CHAN1_TE		(1 << 1)	
#define		MCR_CHAN0_TE		(1 << 0)

#define REG_DMA_STS	0x2AC
#define		DMA_STS_SLICE3_EMPTY	(1 << 31)
#define		DMA_STS_SLICE2_EMPTY	(1 << 30)
#define		DMA_STS_SLICE1_EMPTY	(1 << 29)
#define		DMA_STS_SLICE0_EMPTY	(1 << 28)
#define		DMA_STS_SLICE3_DMADONE	(1 << 27)	
#define		DMA_STS_SLICE2_DMADONE	(1 << 26)
#define		DMA_STS_SLICE1_DMADONE	(1 << 25)
#define		DMA_STS_SLICE0_DMADONE	(1 << 24)
#define		DMA_STS_SLICE3_FEILD	(1 << 23)
#define		DMA_STS_SLICE2_FEILD	(1 << 22)
#define		DMA_STS_SLICE1_FEILD	(1 << 21)
#define		DMA_STS_SLICE0_FEILD	(1 << 20)
#define		DMA_STS_CHAN6_MMUABRT	(1 << 14)
#define		DMA_STS_CHAN5_MMUABRT	(1 << 13)
#define		DMA_STS_CHAN4_MMUABRT	(1 << 12)
#define		DMA_STS_CHAN3_MMUABRT	(1 << 11)
#define		DMA_STS_CHAN2_MMUABRT	(1 << 10)
#define		DMA_STS_CHAN1_MMUABRT	(1 << 9)
#define		DMA_STS_CHAN0_MMUABRT	(1 << 8)
#define		DMA_STS_CHAN6_DMAABRT	(1 << 6)
#define		DMA_STS_CHAN5_DMAABRT	(1 << 5)
#define		DMA_STS_CHAN4_DMAABRT	(1 << 4)
#define		DMA_STS_CHAN3_DMAABRT	(1 << 3)
#define		DMA_STS_CHAN2_DMAABRT	(1 << 2)
#define		DMA_STS_CHAN1_DMAABRT	(1 << 1)
#define		DMA_STS_CHAN0_DMAABRT	(1 << 0)

#define REG_SLICE0_STS	0x2b0
#define REG_SLICE1_STS	0x2b4
#define REG_SLICE2_STS	0x2b8
#define REG_SLICE3_STS	0x2bc
#define		SLICE_STS_VAP_SHIFT	2
#define		SLICE_STS_VAP_MASK	(0xfffffffc)
#define		SLICE_STS_SOURCE	(1 << 1)  /* Valid only for slice 0 */
#define		SLICE_STS_TASK		(1 << 0)

#define REG_AVSTS	0x2C0
#define		AVSTS_FM_STEREO		(1 << 17)
#define		AVSTS_FM_DUAL		(1 << 16)
#define		AVSTS_FM_PILOT		(1 << 15)
#define		AVSTS_NICAM_SMB		(1 << 14)
#define		AVSTS_NICAM_DMB		(1 << 13)
#define		AVSTS_NICAM_VDSP	(1 << 12)
#define		AVSTS_I2C_MASK		(0x3 << 10)
#define		AVSTS_MACROV_MASK 	(0x7 << 7)
#define		AVSTS_FIDT		(1 << 6)
#define		AVSTS_INTL		(1 << 5)
#define		AVSTS_RDCAP		(1 << 4)
#define		AVSTS_PWRON		(1 << 3)
#define		AVSTS_LOADERR		(1 << 2)
#define		AVSTS_TRIGERR		(1 << 1)
#define		AVSTS_CONFERR		(1 << 0)

#define REG_FIEN	0x2C4
#define		FIEN_SLICE3_1 		(1 << 25)
#define		FIEN_SLICE3_0		(1 << 24)
#define		FIEN_SLICE2_T1_OE1	(1 << 19)
#define		FIEN_SLICE2_T1_OE0	(1 << 18)
#define		FIEN_SLICE2_T0_OE1 	(1 << 17)
#define		FIEN_SLICE2_T0_OE0	(1 << 16)
#define		FIEN_SLICE1_T1_OE1	(1 << 19)
#define		FIEN_SLICE1_T1_OE0	(1 << 18)
#define		FIEN_SLICE1_T0_OE1	(1 << 17)
#define		FIEN_SLICE1_T0_OE0	(1 << 16)
#define		FIEN_SLICE0_R1_T1_OE1	(1 << 7)
#define		FIEN_SLICE0_R1_T1_OE0 	(1 << 6)
#define		FIEN_SLICE0_R1_T0_OE1 	(1 << 5)
#define		FIEN_SLICE0_R1_T0_OE0 	(1 << 4)
#define		FIEN_SLICE0_R0_T1_OE1 	(1 << 3)
#define		FIEN_SLICE0_R0_T1_OE0 	(1 << 2)
#define		FIEN_SLICE0_R0_T0_OE1 	(1 << 1)
#define		FIEN_SLICE0_R0_T0_OE0 	(1 << 0)

#define REG_IEN		0x2C8
#define		IEN_DDSTS		(1 << 21)
#define		IEN_DSPMON		(1 << 20)
#define		IEN_EPICS_N		(1 << 19)
#define		IEN_EPICS_P		(1 << 18)
#define		IEN_GPIO_N23		(1 << 17)
#define		IEN_GPIO_P23		(1 << 16)
#define		IEN_GPIO_N22		(1 << 15)
#define		IEN_GPIO_P22		(1 << 14)
#define		IEN_GPIO_P18		(1 << 13)
#define		IEN_GPIO_N18		(1 << 12)
#define		IEN_GPIO_P16		(1 << 11)
#define		IEN_GPIO_N16		(1 << 10)
#define		IEN_LOADERR		(1 << 9)
#define		IEN_CONFERR		(1 << 8)
#define		IEN_TRIGERR		(1 << 7)
#define		IEN_MACROV		(1 << 6)
#define		IEN_FIDT		(1 << 5)
#define		IEN_INTL		(1 << 4)
#define		IEN_RDCAP		(1 << 3)
#define		IEN_PWRON		(1 << 2)
#define		IEN_PCI_PERROR		(1 << 1)
#define		IEN_PCI_AERROR		(1 << 0)

#define	REG_IREP	0x2CC
#define		IREP_DDSP		(1 << 20)
#define		IREP_DSPMON		(1 << 19)
#define		IREP_EPICS		(1 << 18)
#define		IREP_GPIO23		(1 << 17)
#define		IREP_GPIO22		(1 << 16)
#define		IREP_GPIO18		(1 << 15)
#define		IREP_GPIO16		(1 << 14)
#define		IREP_LOADERR		(1 << 13)
#define		IREP_CONFERR		(1 << 12)
#define		IREP_TRIGERR		(1 << 11)
#define		IREP_MACROV		(1 << 10)
#define		IREP_FIDT		(1 << 9)
#define		IREP_INTL		(1 << 8)
#define		IREP_RDCAP		(1 << 7)
#define		IREP_PWRON		(1 << 6)
#define		IREP_PERROR		(1 << 5)
#define		IREP_AERROR		(1 << 4)
#define		IREP_SLICE3		(1 << 3)
#define		IREP_SLICE2		(1 << 2)
#define		IREP_SLICE1		(1 << 1)
#define		IREP_SLICE0		(1 << 0)

#define	REG_ISTS	0x2d0
#define		ISTS_SLICE0_OVRFLOW	(1 << 7)
#define		ISTS_SLICE0_REGION	(1 << 6)
#define		ISTS_SLICE0_TASK	(1 << 5)
#define		ISTS_SLICE0_FEILD	(1 << 4)


/* Video Registers */
#define BREG_INCR_DELAY	0x101
#define		INCR_DELAY_WPOFF	(1 << 6)
#define		DEFAULT_INCR_DELAY	0x08

#define BREG_AINPUT1	0x102
#define		AINPUT1_FUSE_SHIFT	6
#define		AINPUT1_FUSE_MASK	(0x3 << 6)
		enum FUSE {
			FUSE_GAON_AAOFF = (0x10 << 6),
			FUSE_GAON_AAON	= (0x11 << 6),
			FUSE_GAOFF_AAOFF = (0x0 << 6)
		};
#define		AINPUT1_MODE_SHIFT	0
#define		AINPUT1_MODE_MASK	(0xf << 0)
		enum MODE {
			MODE_CVBS_1 = 0,
			MODE_CVBS_2 = 1,
			MODE_CVBS_3 = 2,
			MODE_CVBS_4 = 3,
			MODE_CVBS_5 = 4,
			MODE_SVIDEOFIXGC_1 = 6,
			MODE_SVIDEOFIXGC_2 = 7,
			MODE_SVIDEOAGC_1 = 8,
			MODE_SVIDEOAGC_2 = 9
		};

#define BREG_AINPUT2	0x103
#define		AINPUT2_TEST		(1 << 7)
#define		AINPUT2_HLNRS		(1 << 6)
#define		AINPUT2_VBSL		(1 << 5)
#define		AINPUT2_CPOFF		(1 << 4)
#define		AINPUT2_HOLDG		(1 << 3)
#define		AINPUT2_GAINFIX		(1 << 2)
#define		AINPUT2_GAIN2_BIT8	(1 << 1)
#define		AINPUT2_GAIN1_BIT8	(1 << 0)

#define BREG_AINPUT3_GAIN1	0x104
#define		DEFAULT_GAIN1			0x90

#define BREG_AINPUT4_GAIN2	0x105
#define		DEFAULT_GAIN2			0x90

#define BREG_HGB		0x106
#define		DEFAULT_HGB			0xeb

#define BREG_HGE		0x107
#define		DEFAULT_HGE			0xe0

#define BREG_SYNC_CTRL	0x108
#define		SYNC_CTRL_AUFD		(1 << 7)
#define		SYNC_CTRL_FSEL		(1 << 6)
		enum FRATE {
			FRATE_50 = 50,
			FRATE_60 = 60,
			FRATE_AUTO = 0
		};
#define		SYNC_CTRL_FOET		(1 << 5)
#define		SYNC_CTRL_HTC_SHIFT	3
#define		SYNC_CTRL_HTC_MASK	(0x3 << 3)
		enum HTCMODE {
			HTCMODE_TV = 0x00 << 3,
			HTCMODE_VTR = 0x01 << 3,
			HTCMODE_FAST = 0x11 << 3
		};
#define		SYNC_CTRL_HPLL		(1 << 2)
#define		SYNC_CTRL_VNO_SHIFT	0
#define		SYNC_CTRL_VNO_MASK	(0x3 << 0)
		enum VNOMODE {
			VNOMODE_NORM = 0x00,
			VNOMODE_FAST = 0x01,
			VNOMODE_FREE = 0x02,
			VNOMODE_IMMED = 0x03
		};

#define	BREG_LUMA_CTRL	0x109
#define		LUMA_CTRL_BYPS		(1 << 7)
#define		LUMA_CTRL_YCOMB		(1 << 6)
#define		LUMA_CTRL_LDEL		(1 << 5)
#define		LUMA_CTRL_LUBW		(1 << 4)
#define		LUMA_CTRL_LUFI_SHIFT	0
#define		LUMA_CTRL_LUFI_MASK	(0xf << 0)

#define BREG_LUMA_BRIGHT	0x10a
#define BREG_LUMA_CONT		0x10b
#define BREG_CHROMA_SATN 	0x10c
#define BREG_CHROMA_HUE		0x10d

#define BREG_CHROMA_CTRL1	0x10e
#define 	CHROMA_CTRL1_CDTO	(1 << 7)
#define		CHROMA_CTRL1_CSTD_SHIFT	4
#define		CHROMA_CTRL1_CSTD_MASK	(0x7 << 4)
		enum CSTD {
			CSTD_PAL = 	(0x0 << 4),
			CSTD_NTSC = 	(0x0 << 4),
			CSTD_PAL2 =	(0x4 << 4),
			CSTD_NTSC2 =	(0x4 << 4),
			CSTD_SECAM =	(0x5 << 4),
			CSTD_NTSC3 =	(0x5 << 4)
		};
#define		CHROMA_CTRL1_DCVF	(1 << 3)
#define		CHROMA_CTRL1_FCTC	(1 << 2)
#define		CHROMA_CTRL1_AUTO0	(1 << 1)
#define		CHROMA_CTRL1_CCOMB	(1 << 0)

#define BREG_CHROMA_GAIN	0x10f
#define		CHROMA_GAIN_ACGC	(1 << 7)
#define		CHROMA_GAIN_SHIFT	0
#define		CHROMA_GAIN_MASK	(0x7f << 0)

#define BREG_CHROMA_CTRL2	0x110
#define		CHROMA_CTRL2_OFFU_SHIFT	6
#define		CHROMA_CTRL2_OFFU_MASK	(0x3 << 6)
#define		CHROMA_CTRL2_OFFV_SHIFT	4
#define		CHROMA_CTRL2_OFFV_MASK	(0x3 << 4)
#define		CHROMA_CTRL2_CHBW	(1 << 3)
#define		CHROMA_CTRL2_LCBW_SHIFT	0
#define		CHROMA_CTRL2_LCBW_MASK	(0x7 << 0)

#define BREG_DELAY_CTRL		0x111
#define		DELAY_CTRL_COLO		(1 << 7)
#define		DELAY_CTRL_HDEL_SHIFT	4
#define		DELAY_CTRL_HDEL_MASK	(0x3 << 4)
#define		DELAY_CTRL_YDEL_SHIFT	0
#define		DELAY_CTRL_YDEL_MASK	(0x7 << 0)

#define BREG_ADC_CTRL		0x114
#define		ADC_CTRL_CM99		(1 << 7)
#define		ADC_CTRL_UPTCV		(1 << 6)
#define		ADC_CTRL_AOSL_SHIFT	4
#define		ADC_CTRL_AOSL_MASK	(0x3 << 4)
#define		ADC_CTRL_AUTO1		(1 << 2)
#define		ADC_CTRL_APCK_SHIFT	0
#define		ADC_CTRL_APCK_MASK	(0x3 << 0)

#define BREG_VGB	0x115
#define		DEFAULT_VGB		0x11

#define BREG_VGE	0x116
#define		DEFAULT_VGE		0xfe

#define BREG_MISC_CTRL		0x117
#define		MISC_CTRL_LATY_SHIFT	3
#define		MISC_CTRL_LATY_MASK	(0x7 << 3)
#define		MISC_CTRL_VGPS		(1 << 2)
#define		MISC_CTRL_VGE_BIT8	(1 << 1)
#define		MISC_CTRL_VGB_BIT9	(1 << 0)

#define BREG_VBI_GAIN		0x118
#define		DEFAULT_VBI_GAIN	0x40

#define BREG_VBI_OFFSET		0x119
#define		DEFAULT_VBI_OFFSET	0x80

#define BREG_VIDEO_STS1		0x11e
#define		VIDEO_STS1_HLCK		(1 << 6)
#define		VIDEO_STS1_SLTCA		(1 << 5)
#define		VIDEO_STS1_GLIMT		(1 << 4)
#define		VIDEO_STS1_GLIMB		(1 << 3)
#define		VIDEO_STS1_WIPA		(1 << 2)
#define		VIDEO_STS1_DCSTD_SHIFT	0
#define		VIDEO_STS1_DCSTD_MASK	(0x3 << 0)

#define BREG_VIDEO_STS2		0x11f
#define		VIDEO_STS2_INTL		(1 << 7)
#define		VIDEO_STS2_HLVLN		(1 << 6)
#define		VIDEO_STS2_FIDT		(1 << 5)
#define		VIDEO_STS2_TYPE3		(1 << 3)
#define		VIDEO_STS2_COLSTR		(1 << 2)
#define		VIDEO_STS2_COPRO		(1 << 1)
#define		VIDEO_STS2_RDCAP		(1 << 0)

enum		ADETECT {
			ADETECT_DISABLE = 0,
			ADETECT_LEVEL3 = 1,
			ADETECT_LEVEL2 = 2,
			ADETECT_LEVEL1 = 3
		};


/* Scaler Registers */
#define BREG_SRC_TMNG1				0x00
#define		SRC_TMNG1_YPOS_LOBITS_SHIFT	0
#define		SRC_TMNG1_YPOS_LOBITS_MASK	(0xff << 0)

#define BREG_SRC_TMNG2		0x01
#define		SRC_TMNG2_DFID			(1 << 6)
#define		SRC_TMNG2_DVED			(1 << 5)
#define		SRC_TMNG2_DHED			(1 << 4)
#define		SRC_TMNG2_YPOS_HIBITS_SHIFT	0
#define		SRC_TMNG2_YPOS_HIBITS_MASK 	(0xf << 0)

#define	BREG_REGION_EN		0x04
#define		REGION_EN_SWRST		(1 << 7)
#define		REGION_EN_VBI_B		(1 << 5)
#define		REGION_EN_VIDEO_B		(1 << 4)
#define		REGION_EN_VBI_A		(1 << 1)
#define		REGION_EN_VIDEO_A		(1 << 0)

#define BREG_SCALER_STS1	0x06
#define		SCALER_STS1_VBI_B	(1 << 5)
#define		SCALER_STS1_VIDEO_B	(1 << 4)
#define		SCALER_STS1_VBI_A	(1 << 1)
#define		SCALER_STS1_VIDEO_A	(1 << 0)

#define BREG_SCALER_STS0	0x07
#define		SCALER_STS2_TASK	(1 << 7)
#define		SCALER_STS2_D5XORD4	(1 << 6)
#define		SCALER_STS2_FIDSCO	(1 << 5)
#define		SCALER_STS2_FIDSCI	(1 << 4)
#define		SCALER_STS2_WASRST	(1 << 3)
#define		SCALER_STS2_LDERR	(1 << 2)
#define		SCALER_STS2_CFERR	(1 << 1)
#define		SCALER_STS2_TRERR	(1 << 0)

#define	BREG_LUT_GSPOS		0x0c
#define	BREG_LUT_BSPOS		0x0d
#define	BREG_LUT_RSPOS		0x0e

#define BREG_LUT_GPOS(x)	((x & 0xf) + 0x010)
#define BREG_LUT_BPOS(x)	((x & 0xf) + 0x020)
#define BREG_LUT_RPOS(x)	((x & 0xf) + 0x030)


/* Task Registers */
#define	BREG_TASK_COND(t)	(0x00 + (t ? 0x80 : 0x40))
#define		TASK_COND_SLCK		(1 << 4)
#define		TASK_COND_U60		(1 << 3)
#define		TASK_COND_U50		(1 << 2)
#define		TASK_COND_UEVEN		(1 << 1)
#define		TASK_COND_UODD		(1 << 0)

#define	BREG_TASK_FEILD(t)	(0x01 + (t ? 0x80 : 0x40))
#define		TASK_FEILD_FSKIP_SHIFT	2
#define		TASK_FEILD_FSKIP_MASK	(0x3f << 2)
#define		TASK_FEILD_COUNT_SHIFT	0
#define		TASK_FEILD_COUNT_MASK	(0x3 << 0)

#define BREG_TASK_DPATH(t)	(0x02 + (t ? 0x80 : 0x40))
#define		TASK_DPATH_VBI_RAWS_SHIFT	6
#define		TASK_DPATH_VBI_RAWS_MASK	(0x3 << 6)
		enum VBIRAWSRC { 
			VBIRAWSRC_YUV =		(0x0 << 6),
			VBIRAWSRC_ADC1_LUMA =	(0x2 << 6),
			VBIRAWSRC_ADC2_LUMA =	(0x3 << 6)
		};
#define		TASK_DPATH_FIDC			(1 << 5)
#define		TASK_DPATH_GPSEL		(1 << 4)
#define		TASK_DPATH_PCSEL		(1 << 3)
#define		TASK_DPATH_GACON		(1 << 2)
#define		TASK_DPATH_CSCON		(1 << 1)
#define		TASK_DPATH_MACON		(1 << 0)

#define	WREG_TASK_VBI_XINSTART(t)	(0x04 + (t ? 0x80 : 0x40))
#define	WREG_TASK_VBI_XINSTOP(t)	(0x06 + (t ? 0x80 : 0x40))
#define	WREG_TASK_VBI_YINSTART(t)	(0x08 + (t ? 0x80 : 0x40))
#define	WREG_TASK_VBI_YINSTOP(t)	(0x0a + (t ? 0x80 : 0x40))
#define WREG_TASK_VBI_XOUTLEN(t)	(0x0c + (t ? 0x80 : 0x40))
#define WREG_TASK_VBI_YOUTLEN(t)	(0x0e + (t ? 0x80 : 0x40))

#define	WREG_TASK_VIDEO_XINSTART(t)	(0x14 + (t ? 0x80 : 0x40)) 	
#define	WREG_TASK_VIDEO_XINSTOP(t)	(0x16 + (t ? 0x80 : 0x40)) 	
#define	WREG_TASK_VIDEO_YINSTART(t)	(0x18 + (t ? 0x80 : 0x40))
#define	WREG_TASK_VIDEO_YINSTOP(t)	(0x1a + (t ? 0x80 : 0x40))
#define WREG_TASK_VIDEO_XOUTLEN(t)	(0x1c + (t ? 0x80 : 0x40))
#define WREG_TASK_VIDEO_YOUTLEN(t)	(0x1e + (t ? 0x80 : 0x40))

#define BREG_TASK_XPSC(t)		(0x20 + (t ? 0x80 : 0x40))
#define BREG_TASK_XPSC_XACL(t)		(0x21 + (t ? 0x80 : 0x40)) 	

#define BREG_TASK_XPSC_CTRL(t)		(0x22 + (t ? 0x80 : 0x40))
#define		TASK_XPSC_CTRL_WEIGHT		(1 << 3)
#define		TASK_XPSC_CTRL_XDCG_SHIFT	0
#define		TASK_XPSC_CTRL_XDCG_MASK	(0x7 << 0)

#define BREG_TASK_FIRFLTR_CTRL(t)	(0x23 + (t ? 0x80 : 0x40))
#define 	TASK_FIRFLTR_RPFC_SHIFT		6
#define 	TASK_FIRFLTR_RPFC_MASK		(0x3 << 6)
#define		TASK_FIRFLTR_RPFY_SHIFT		4
#define		TASK_FIRFLTR_RPFY_MASK		(0x3 << 4)
#define		TASK_FIRFLTR_VPFC_SHIFT		2
#define		TASK_FIRFLTR_VPFC_MASK		(0x3 << 2)
#define		TASK_FIRFLTR_VPFY_SHIFT		0
#define		TASK_FIRFLTR_VPFY_MASK		(0x3 << 0)

#define BREG_TASK_LUMA_BRIGHT(t)	(0x24 + (t ? 0x80 : 0x40))
#define BREG_TASK_LUMA_CONT(t)		(0x25 + (t ? 0x80 : 0x40))
#define BREG_TASK_CHROMA_SATN(t)	(0x26 + (t ? 0x80 : 0x40))

#define WREG_TASK_VBI_XSC(t)		(0x28 + (t ? 0x80 : 0x40))
#define BREG_TASK_VBI_LUMA_PHOFF(t)	(0x2a + (t ? 0x80 : 0x40))
#define BREG_TASK_VBI_CHROMA_PHOFF(t)	(0x2b + (t ? 0x80 : 0x40)) 	

#define WREG_TASK_VIDEO_XSC(t)		(0x2c + (t ? 0x80 : 0x40))
#define BREG_TASK_VIDEO_LUMA_PHOFF(t)	(0x2e + (t ? 0x80 : 0x40))
#define BREG_TASK_VIDEO_CHROMA_PHOFF(t)	(0x2f + (t ? 0x80 : 0x40))

#define WREG_TASK_YSC(t)		(0x30 + (t ? 0x80 : 0x40))
#define BREG_TASK_YFLTR_CNTRL(t)	(0x32 + (t ? 0x80 : 0x40))
#define		TASK_YFLTR_CNTRL_YMIR		(1 << 1)
#define		TASK_YFLTR_CNTRL_YMODE		(1 << 2)

#define BREG_TASK_VPHOFF0(t)		(0x34 + (t ? 0x80 : 0x40))
#define BREG_TASK_VPHOFF1(t)		(0x35 + (t ? 0x80 : 0x40))
#define BREG_TASK_VPHOFF2(t)		(0x36 + (t ? 0x80 : 0x40)) 	
#define BREG_TASK_VPHOFF3(t)		(0x37 + (t ? 0x80 : 0x40)) 	


/* Clipping & Output Registers */
#define BREG_TASK_VIDEO_FMT(t)		(0x300 + (t ? 2 : 0))
#define BREG_TASK_VBI_FMT(t)		(0x301 + (t ? 2 : 0))

#define BREG_TASK_VIDEO_CLIPMODE(t)	(0x300 + (t ? 2 : 0))
#define BREG_TASK_VBI_CLIPMODE(t)	(0x301 + (t ? 2 : 0))

		enum CLIPMODE { 
			CLIPMODE_OFF =		(0 << 5),
			CLIPMODE_RECT =		(1 << 5),
			CLIPMODE_ALPHA =	(2 << 5),
			CLIPMODE_FIXED =	(3 << 5),
		};

#define BREG_CLIP_ANCL		0x304
#define BREG_CLIP_ACL		0x305

#define BREG_VIDEO_UVL		0x308
#define		VIDEO_UVL_SHIFT			0
#define		VIDEO_UVL_MASK			(3 << 0)
#define		VIDEO_UVL_CLINV			(1 << 2)	
#define		VIDEO_UVL_INTLCD		(1 << 3)	

#define BREG_CLIP_CLCR		0x309
#define BREG_CLIP_CLCG		0x30a
#define BREG_CLIP_CLCB		0xc0b

#define REG_CLIP_X(i)		(0x380 + i * 8)
#define REG_CLIP_Y(i)		(0x384 + i * 8)

/* Audio Registers SAA7134 specific */
#define WREG_NICAMDATA		0x140
#define		NICAMDATA_AUXDATA_SHIFT		0
#define		NICAMDATA_AUXDATA_MASK		(0x7ff << 0)
#define		NICAMDATA_CI_SHIFT		11
#define		NICAMDATA_CI_MASK		(0x3 << 11)
#define 	NICAMDATA_SAD			(1 << 14)
#define 	NICAMDATA_OVW			(1 << 15)

#define BREG_NICAMSTS		0x142
#define		NICAMSTS_SIN_SHIFT		0
#define		NICAMSTS_SIN_MASK		(0x3 << 0)
#define 	NICAMSTS_CFC			(1 << 2)
#define		NICAMSTS_C0S			(1 << 3)
#define		NICAMSTS_C_SHIFT		4 
#define		NICAMSTS_C_MASK			(0xf << 4)

#define BREG_AUDSTS		0x143
#define		AUDSTS_VDSP			(1 << 1)
#define		AUDSTS_AMSTS			(1 << 2)

#define BREG_NICAMERRCNT	0x144

#define BREG_IDENT		0x145
#define		IDENT_SIFL_SHIFT		0
#define		IDENT_SIFL_MASK			(0x1f << 0)
#define		IDENT_IDP_SHIFT			5
#define		IDENT_IDP_MASK			(0x7 << 5)
#define		IDENT_IDP_CHAN2_VALID		(1 << 5)
#define		IDENT_IDP_2LANG_SIGNAL		(1 << 6)
#define		IDENT_IDP_STEREO_SIGNAL		(1 << 7)

#define WREG_MON_RDOUT		0x146
#define BREG_NICAMERR_LLIMIT	0x148
#define BREG_NICAMERR_ULIMIT	0x149

#define BREG_DXCO_CTRL		0x14a
#define		DXCO_CTRL_IDAREA		(1 << 0)
#define		DXCO_CTRL_IDMOD_SHIFT		1
#define		DXCO_CTRL_IDMOD_MASK		(0x3 << 1)
		enum IDMOD {
			IDMOD_SLOW = 0x0 << 1,
			IDMOD_MEDIUM = 0x1 << 1,
			IDMOD_FAST = 0x2 << 1,
			IDMOD_OFF = 0x3 << 1
		};
#define 	DXCO_CTRL_TSTC_SHIFT		4
#define 	DXCO_CTRL_TSTC_MASK		(0x3 << 4)

#define BREG_DMOD_CFG		0x14b
#define		DMOD_CFG_CH2_MODE_0		(1 << 0)
		/* Enum special handling required as CH2_MODE 
		 * is split into _0 and _1 
		 */
		enum CH2MODE {
			CH2MODE_FM = 0x0 << 0,
			CH2MODE_AM = 0x1 << 0,
			CH2MODE_NICAM = 0x2 << 0
		};
#define		DMOD_CFG_CH1_MODE		(1 << 1)
		enum CH1MODE {
			CH1MODE_FM = 0x0 << 1,
			CH1MODE_AM = 0x1 << 1
		};
#define		DMOD_CFG_FLTBW_SHIFT		2
#define		DMOD_CFG_FLTBW_MASK		(0x3 << 2)
		enum FLTBW {
			FLTBW_C1MIN_C2MIN = 0x0 << 2,
			FLTBW_C1MAX_C2MIN = 0x1 << 2,
			FLTBW_C1NORM_C2NORM = 0x2 << 2,
			FLTBW_C1MAX_C2MAX = 0x3 << 2
		};
#define		DMOD_CFG_CH2_MODE_1_SHIFT	4
#define		DMOD_CFG_CH2_MODE_1		(1 << 4)

#define BREG_AUDGAIN		0x14c
#define 	AUDGAIN_SIFG_SHIFT		0
#define		AUDGAIN_SIFG_MASK		(0x1f << 0)
#define 	AUDGAIN_AGC_SLOW		(1 << 5)
#define 	AUDGAIN_AGC_OFF			(1 << 6)

#define REG_CAR1FREQ		0x150
#define REG_CAR2FREQ		0x154
#define WREG_AUDDMACNT_0_15	0x158
#define BREG_AUDDMACNT_16_21	0x15a

#define BREG_AUDFMT		0x15b
#define		AUDFMT_FMT_SHIFT		0
#define		AUDFMT_FMT_MASK			(0x3 << 0)
		enum AUDFMT {
			AUDFMT_8BIT = 0x0 << 0,
			AUDFMT_16BIT = 0x1 << 0,
			AUDFMT_20BIT_LALIGN = 0x2 << 0,
			AUDFMT_20BIT_RALIGN = 0x3 << 0
		};
#define		AUDFMT_FLAG_2COMPLEMENT		(1 << 2)
#define		AUDFMT_SAMP_SHIFT		3
#define		AUDFMT_SAMP_MASK		(0x7 << 3)
		enum AUDSAMP {
			/* XXX Manual defines STEREO as 0x1 which is 
			 * incorrect, furthermore it does not mention 
			 * AUDSAMP_MONO_LEFT, AUDSAMP_MONO_RIGHT, 
			 * these have been discovered via experimentation
			 */
			AUDSAMP_MONO_LEFT = 0x1 << 3,
			AUDSAMP_MONO_RIGHT = 0x2 << 3,
			AUDSAMP_STEREO = 0x3 << 3,
			AUDSAMP_LEFT_LEFT = 0x5 << 3,
			AUDSAMP_RIGHT_RIGHT = 0x6 << 3
		};
#define		AUDFMT_AUSEL_SHIFT		6
#define		AUDFMT_AUSEL_MASK		(0x3 << 6)
		enum AUDSEL {
			AUDSEL_BBAND = 0x2 << 6,
			AUDSEL_SIF = 0x3 << 6
		};

#define BREG_MONSEL		0x160
#define		MONSEL_MSS_SHIFT		0
#define		MONSEL_MSS_MASK			(0x3 << 0)
		enum MSS {
			MSS_DC_OUT = 0x0 << 0,
			MSS_PHASE_OUT = 0x1 << 0,
			MSS_FMAMPATH_OUT = 0x2 << 0,
			MSS_NICAMPATH_OUT = 0x3 << 0
		};
#define 	MONSEL_MCSM_SHIFT		4
#define 	MONSEL_MCSM_MASK		(0x3 << 4)
		enum MCSM {
			MCSM_AVG = 0x0 << 4,
			MCSM_CHAN1 = 0x1 << 4,
			MCSM_CHAN2 = 0x2 << 4
		};
#define 	MONSEL_PEAK			(1 << 7)

#define BREG_FMDEMP		0x161
#define		FMDEMP_DSC1_SHIFT		0
#define		FMDEMP_DSC1_MASK		(0x7 << 0)
		enum FMDSC1 {
			FMDSC1_50us = 0x0 << 0,
			FMDSC1_60us = 0x1 << 0,
			FMDSC1_75us = 0x2 << 0,
			FMDSC1_J17 = 0x3 << 0,
			FMDSC1_OFF = 0x7 << 0
		};
#define		FMDEMP_ADEMP1			(1 << 3)
#define		FMDEMP_DSC2_SHIFT		4
#define		FMDEMP_DSC2_MASK		(0x7 << 4)
		enum FMDSC2 {
			FMDSC2_50us = 0x0 << 0,
			FMDSC2_60us = 0x1 << 0,
			FMDSC2_75us = 0x2 << 0,
			FMDSC2_J17 = 0x3 << 0,
			FMDSC2_OFF = 0x7 << 0
		};
#define		FMDEMP_ADEMP2			(1 << 7)

#define BREG_FMDMAT		0x162
#define		FMDMAT_FMDMS_SHIFT		0
#define		FMDMAT_FMDMSEL_MASK		(0x7 << 0)
		enum FMDMSEL {
			FMDMSEL_MONOCH1 = 0x0 << 0,
			FMDMSEL_MONOCH2 = 0x1 << 0,
			FMDMSEL_DUAL = 0x2 << 0,
			FMDMSEL_DUALSWAP = 0x3 << 0,
			FMDMSEL_STEREOEUROPE = 0x4 << 0,
			FMDMSEL_STEREOKOREA6 = 0x5 << 0,
			FMDMSEL_STEREOKOREA = 0x6 << 0
		};
#define 	FMDMAT_IDSWFM			(1 << 7)

#define BREG_CH1_LVLADJ		0x163		
#define BREG_CH2_LVLADJ		0x164		

#define BREG_NICAMCFG		0x165		
#define 	NICAMCFG_AMUTE			(1 << 0)
#define 	NICAMCFG_NDEEM			(1 << 1)
#define 	NICAMCFG_AMSEL			(1 << 2)

#define BREG_NICAMLVL		0x166		

#define BREG_DACOSEL		0x167
#define 	DACOSEL_SDOS_SHIFT		0
#define 	DACOSEL_SDOS_MASK		(0x3 << 0)
		enum SDOS {
			SDOS_FMAM_FMAM = 0x0 << 0,
			SDOS_NICAM_NICAM = 0x1 << 0,
			SDOS_FMAM_NICAMM1 = 0x2 << 0,
			SDOS_FMAM_NICAMM2 = 0x3 << 0
		};
#define 	DACOSEL_AVL_SHIFT		4
#define 	DACOSEL_AVL_MASK		(0x3 << 4)
		enum AVL {
			AVL_OFF = 0x0 << 4,
			AVL_SHORT = 0x1 << 4,
			AVL_MEDIUM = 0x2 << 4,
			AVL_LONG = 0x3 << 4
		};
#define 	DACOSEL_SDGS_SHIFT		6
#define 	DACOSEL_SDGS_MASK		(0x3 << 6)
		enum SDGS {
			SDGS_0DB = 0x0 << 6,
			SDGS_3DB = 0x1 << 6,
			SDGS_6DB = 0x2 << 6,
			SDGS_9DB = 0x3 << 6
		};

#define BREG_I2SOFMT		0x168
#define		I2SOFMT_SOUT			(1 << 0)
#define		I2SOFMT_SFMT			(1 << 1)

#define BREG_I2SOSEL		0x169
#define BREG_I2SOLVL		0x16a

#define BREG_DSPOSEL		0x16b
#define		DSPOSEL_CSM_SHIFT		4
#define		DSPOSEL_CSM_MASK		(0x7 << 4)
		enum CSM {
			CSM_LEFT_RIGHT = 0x0 << 4,
			CSM_LEFT_LEFT = 0x1 << 4,
			CSM_RIGHT_RIGHT = 0x2 << 4,
			CSM_RIGHT_LEFT = 0x3 << 4,
			CSM_AVG = 0x4 << 4
		};
#define 	DSPOSEL_AASDMA			(1 << 7)

#define BREG_AUDMUTE		0x16c
#define		AUDMUTE_SOUT			(1 << 3)
#define		AUDMUTE_I2S			(1 << 6)

#define BREG_SFREQ		0x16d
#define		SFREQ_SFS_SHIFT			0
#define		SFREQ_SFS_MASK			(0x3 << 0)
		enum SFS {
			SFS_SIF_32KHZ = 0x0 << 0,
			SFS_BBAND_32KHZ = 0x1 << 0,
			SFS_BBAND_44KHZ = 0x2 << 0,	/* 44100 KHz */
			SFS_BBAND_48KHZ = 0x3 << 0
		};
#define		SFREQ_HPEN			(1 << 2)
#define 	SFREQ_SIFP_SHIFT		4
#define 	SFREQ_SIFP_MASK			(0x3 << 4)
#define 	SFREQ_SIFA_SHIFT		6
#define 	SFREQ_SIFA_MASK			(0x3 << 6)
		enum SIFA {
			SIFA_0DB = 0x0 << 6,
			SIFA_N6DB = 0x1 << 6,
			SIFA_N10DB = 0x2 << 6
		};

#define BREG_AUDIOSEL		0x16e
#define		AUDIOSEL_OCS_SHIFT		0
#define		AUDIOSEL_OCS_MASK		(0x7 << 0)
		enum OCS {
			OCS_INLEFT1_INRIGHT1 = 0x0 << 0,
			OCS_INLEFT2_INRIGHT2 = 0x1 << 0,
			OCS_DACLEFT_DACRIGHT = 0x2 << 0,
			OCS_INLEFT1_INLEFT1 = 0x3 << 0,
			OCS_INRIGHT1_INRIGHT1 = 0x4 << 0,
			OCS_INLEFT2_INLEFT2 = 0x5 << 0,
			OCS_INRIGHT2_INRIGHT2 = 0x6 << 0
		};
#define 	AUDIOSEL_ICS			(1 << 3)
#define		AUDIOSEL_VSEL1_2VRMS		(1 << 4)
#define		AUDIOSEL_VSEL2_2VRMS		(1 << 5)

#define WREG_ACNI_0_15		0x170
#define BREG_ACNI_16_21		0x172

#define BREG_APLLCTRL		0x173
#define		APLLCTRL_SWLOOP			(1 << 0)
#define		APLLCTRL_APLL			(1 << 1)

#define WREG_ACPF_0_15		0x174
#define BREG_ACPF_16_17		0x176


/* I2C Registers */
#define BREG_I2C_TIMER		0x183
#define		I2C_TIMER_SHIFT		4
#define		I2C_TIMER_MASK		(0xf << 4)
#define		I2C_TIMER_DISABLED	0

#define BREG_I2C_CLKSEL		0x182
#define		I2C_CLKSEL_FAST		(1 << 6)

#define	BREG_I2C_DATA		0x181

#define	BREG_I2C_CTL		0x180
#define		I2C_CTL_ATTR_SHIFT	6
#define		I2C_CTL_ATTR_MASK	(0x3 << 6)
#define		I2C_CTL_STS_SHIFT	0
#define		I2C_CTL_STS_MASK	(0x0f << 0)


/* SAA7135 Audio Registers */
#define BREG_S5A_DSPSTAT	0x586
#define		DSPSTAT_RRDR		(1 << 1)	
#define		DSPSTAT_RERR		(1 << 1)	

#define BREG_S5A_EPICSSTAT	0x580
#define		EPICSSTAT_ERR		(1 << 3)
#define		EPICSSTAT_IDA		(1 << 2)
#define		EPICSSTAT_RDR		(1 << 1)
#define		EPICSSTAT_WRR		(1 << 0)


/* 
 * SAA713x GPIO Registers, Valid pins: 0-23, 24-27 
 */
#define	REG_GPIO_DIR		0x1b0	/* set GPIO direction: 0=in, 1=out */
#define		GPIO_READ_LATCH		(1 << 31)

#define	REG_GPIO_DATA		0x1b4	/* r/w GPIO pins */

#endif
