#include <sys/param.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>
#include <fcntl.h>

#include <Fl/Fl_Window.h>
#include <Fl/Fl_Button.h>
#include <Fl/Fl.h>

#include "/sys/dev/iicbus/iic.h"

extern "C" {
#include "tuner_ctrl.h"
}

#define TVVRC_OFFSET	4
#define MAX_CHANNELS	128


float chanlist[MAX_CHANNELS];
int cur_chan, max_chan;
int dfd;

static
void cb_prevchan(Fl_Widget*, void*) 
{
	int rv, c;

	c = cur_chan;
	if (--cur_chan < 0)
		cur_chan = max_chan - 1;

	rv = set_channel(dfd, chanlist[cur_chan], chanlist[c], 0);
	if (rv != 0)
		cur_chan = c;
}

static
void cb_nextchan(Fl_Widget*, void*) 
{
	int rv, c;

	c = cur_chan;
	if (++cur_chan >= max_chan)
		cur_chan = 0;

	rv = set_channel(dfd, chanlist[cur_chan], chanlist[c], 0);
	if (rv != 0)
		cur_chan = c;
}


static void
create_chanlist(FILE *fp)
{
	int i;
	float f;
	char buf[1024];

	fseek(fp, TVVRC_OFFSET, SEEK_SET);
	for (i = 0; i < MAX_CHANNELS;) {
		fscanf(fp, "%s\n", buf);
		if (sscanf(buf, "%f\n", &f) == 1) {
			chanlist[i++] = f;
		}
		if (feof(fp))
			break;
	}
	if (i == 0)
		errx(ENOTSUP, "error, no channels defined");
	max_chan = i;
}


int 
main (int argc, char **argv) 
{
	FILE *fp;

	if (argc != 3)
		errx(ENOTSUP, "usage: %s <iic_device> <channel_filename>",
			argv[0]);

	if ( (dfd = open(argv[1], O_RDWR, 0)) == -1)
		err(errno, "error opening %s", argv[1]);

	if ( (fp = fopen(argv[2], "r")) == NULL)
		err(errno, "error opening %s", argv[2]);

	create_chanlist(fp);
	cur_chan = 0;
	set_channel(dfd, chanlist[cur_chan], 0, 0);

	Fl_Window* w;

	w = new Fl_Window(0, 0, 30, 15);
	w->begin();
	{
		Fl_Button* o = new Fl_Button(0, 0, 15, 15, "<-");
		o->callback(cb_prevchan);
	}
	{
		Fl_Button* o = new Fl_Button(15, 0, 15, 15, "->");
		o->callback(cb_nextchan);
	}
	w->end();

	argc = 1;
	w->position(0, 0);
	w->show(argc, argv);

	return Fl::run();
}
