#include <sys/param.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <errno.h>

#include "/sys/dev/iicbus/iic.h"

#include "tuner_ctrl.h"

int
set_channel(int fd, float new_freq, float old_freq, uint8_t *lockval)
{	
	struct timespec tspc;
	struct iiccmd cmd;
	int rval, ectr;
	uint16_t smf;
	uint8_t buf[7], pb;

	if(new_freq >= HIBAND_MINFREQ && new_freq <= HIBAND_MAXFREQ) {
		pb = HIBAND_CMD;
	} else if (new_freq >= MIDBAND_MINFREQ && new_freq <= MIDBAND_MAXFREQ) {
		pb = MIDBAND_CMD;
	} else if (new_freq >= LOBAND_MINFREQ && new_freq <= LOBAND_MAXFREQ) {
		pb = LOBAND_CMD;
	} else {
		return EINVAL;
	}
	smf = (new_freq + PCARRIER_FREQ) * 16;

	if (old_freq < new_freq) {
		buf[0] = (smf >> 8) & 0x7f;
		buf[1] = smf & 0xff;
		buf[2] = CTRL_SIGNATURE | CTRL_STEP_62p5;
		buf[3] = pb;
	} else {
		buf[0] = CTRL_SIGNATURE | CTRL_STEP_62p5;
		buf[1] = pb;
		buf[2] = (smf >> 8) & 0x7f;
		buf[3] = smf & 0xff;
	}

	cmd.slave = TUNER_WR_I2CADDR;
	cmd.count = 1;

	ectr = 1;

retry_post_reset:

	if ( (rval = ioctl(fd, I2CSTART, &cmd)) != 0) {
		if (errno == EINVAL || errno == EILSEQ) {
			usleep(1000);
			ioctl(fd, I2CSTOP, &cmd);
			usleep(1000);
			ioctl(fd, I2CRSTCARD, &cmd);
			usleep(1000);
			if (ectr--)
				goto retry_post_reset;
		}
		return rval;
	}

	cmd.slave = TUNER_WR_I2CADDR;
	cmd.count = 4;
	cmd.buf = buf; 
	cmd.last = 1;
	if ( (rval = ioctl(fd, I2CWRITE, &cmd)) != 0)
		return rval;

	if (!lockval) {
		if ( (rval = ioctl(fd, I2CSTOP)) != 0)
			return rval;
		return 0;
	}

	tspc.tv_sec = 0;
	tspc.tv_nsec = 100000000;
	nanosleep(&tspc, 0);

	if ( (rval = ioctl(fd, I2CSTOP)) != 0)
		return rval;

        /* Read 1 Bytes */  
        cmd.slave = TUNER_RD_I2CADDR;
        cmd.count = 1;
        cmd.buf = 0; 
        cmd.last = 0;
        buf[0] = buf[1] = buf[2] = buf[3] = 0;

        if ( (rval = ioctl(fd, I2CSTART, &cmd)) != 0)
		return rval;

        cmd.slave = TUNER_RD_I2CADDR;
        cmd.count = 1;
        cmd.buf = buf; 
        cmd.last = 0;

        if ( (rval = ioctl(fd, I2CREAD, &cmd)) != 0)
		return rval;

        if ( (rval = ioctl(fd, I2CSTOP)) != 0)
		return rval;

	*lockval = buf[0];
	return 0;
}
