extern "C" {
#include <sys/param.h>
#include <sys/soundcard.h>
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include "saa713x_ioctl.h"

extern FILE* yyin;
}

#define MIXER_PATH	"/dev/mixer"
#define MAX_CHANSTR	32

#include <Fl/fl_ask.H>
#include <Fl/Fl_Valuator.H>
#include <Fl/Fl_Choice.H>
#include <Fl/Fl_Box.H>
#include <Fl/Fl_Scrollbar.H>
#include <Fl/fl_ask.h>

#include <vector>
#include "tvv.h"
#include "tune_channels.h"
#include "set_channel.h"
#include "cntrl.h"

using namespace std;

int *current_channel;

struct GLOBAL {
	vector<struct cinfo>	carray;
	Fl_Choice		*cchoice;
	Fl_Box			*cdpybox;
	Fl_Box			*vdpybox;
	Fl_Box			*brightdpybox;
	Fl_Box			*contdpybox;
	Fl_Box			*satdpybox;
	Fl_Box			*huedpybox;

	Fl_Progress		*tuneprog;
	
	int			chan_lastidx;
	int			vol_lastval;
	int			vol_muteval;
	int			bright_lastval;
	int			cont_lastval;
	int			sat_lastval;
	int			hue_lastval;

	enum VSTD		vstd_last;

	int			saa_fd;
} G;

void
cntrl_set_volume(Fl_Scrollbar *w, int value)
{
	static char vol_dpy_buf[16];
	static int mixer_fd = -2;
	int v;

	if (mixer_fd == -1)		/* XXX Incorrect */
		return;

	if (mixer_fd == -2) {
		mixer_fd = open(MIXER_PATH, O_RDWR, 0);
		if (mixer_fd == -1) {
			fl_message("Error opening mixer: %s: %s", 
					MIXER_PATH, strerror(errno));
			return;
		}
	}

	if (ioctl(mixer_fd, SOUND_MIXER_READ_VOLUME, &v) == -1) {
		fl_message("Mixer IOCTL error: %s", strerror(errno));
	}

	if (v == G.vol_lastval) {
		v = (int) value;
		v |= v << 8;
		if (ioctl(mixer_fd, SOUND_MIXER_WRITE_VOLUME, &v) == -1) {
			fl_message("Mixer IOCTL error: %s", strerror(errno));
		}
	} else {
		if (w)
			w->value(v & 0xff, 0, 0, 100);
	}
	G.vol_lastval = v;
	snprintf(vol_dpy_buf, 32, "%3d%%", G.vol_lastval & 0xff);
	if (G.vdpybox)
		G.vdpybox->label(vol_dpy_buf);
}


void
cntrl_sc_in(int v, Fl_Widget *w) 
{
	static char buf[8];
	const char *s;
	int i;

	if (v < 10) {		/* Kepad Digit */
		i = 0;
		s = w->label();
		if ( s && (i = strlen(s)) >= 3) {
			buf[3] = 0;
			buf[0] = s[1];
			buf[1] = s[2];
			buf[2] = '0' + v;
		} else {
			switch (i) {
			case 2:
				buf[3] = 0;
				buf[2] = '0' + v;
				buf[1] = s[1];
				buf[0] = s[0];
				break;
			case 1:
				buf[2] = 0;
				buf[1] = '0' + v;
				buf[0] = s[0];
				break;
			case 0:
			default:
				buf[1] = 0;
				buf[0] = '0' + v;
				break;
			}
		}
	} else {		/* Keypad Function */
		if (v == 0xa) {		/* Clear Func */
			buf[0] = 0;
		} else if (v == 0xb) {	/* Go Func */
			i = atoi(w->label());
			--i;
			cntrl_channel_set(i);
			buf[0] = 0;
		}
	}
	w->label(buf);
}

void
cntrl_add_channel(double freq, const char *s, int flag)
{
	struct cinfo ci;
	int i;

	ci.ci_name_flag = flag;

	for (i = 0; i < G.carray.size(); ++i)
		if (G.carray[i].ci_freq == freq) {
			warnx("Duplicate frequency: %s", s);
			return;
		}
	ci.ci_freq = freq;

	for (i = 0; i < G.carray.size(); ++i)
		if (strcmp(G.carray[i].ci_name, s) == 0) {
			warnx("Duplicate name: %s", s);
			return;
		}
	strncpy(ci.ci_name, s, MAX_CHANSTR);

	G.carray.push_back(ci);
}


void
cntrl_add_channel_str(const char *s)
{
	char buf[128];
	double freq;
	char *nm;
	int flag;

	freq = 0;
	freq = strtod(s, &nm);
	if (freq != 0) {
		while(nm[0] && isspace(nm[0]))
			++nm;
	
		flag = 0;
		if (strlen(nm) == 0) {
			snprintf(buf, 128, "CHANNEL %d", G.carray.size() + 1);
			nm = buf;
			flag = 1;
		}

		cntrl_add_channel(freq, nm, flag);
	}
}


static void
cntrl_channel_choicecb(Fl_Widget *w, void *priv)
{
	Fl_Choice *c = (Fl_Choice*)w;

	cntrl_channel_set(c->value());
}


void
cntrl_channel_choiceinit(Fl_Choice *c)
{
	int i;

	G.cchoice = c;
	c->clear();
	for (i = 0; i < G.carray.size(); ++i) {
		c->add(G.carray[i].ci_name, 0, cntrl_channel_choicecb);
	}
	

}


void
cntrl_channel_set(int n)
{
	static char chan_dpy_buf[32];
	int rval, dummy; 

	if (n >= 0 && n < G.carray.size()) {
#if XXX_DISABLED_NEEDS_MORE_WORK
		/* VIDEDMACONT is called by the status change signal
		 * handler
		 */
		if (ioctl(G.saa_fd, VIDEODMAPAUSE, &dummy) != 0)
			warn("DMA pause failed");
		cntrl_audio_mute();
		if ( (rval = set_channel_internal(G.carray[n].ci_freq)) != 0) {
			cntrl_audio_demute();
			if (ioctl(G.saa_fd, VIDEODMACONT, &dummy) != 0) {
					warn("DMA continue failed");
			}
			if (rval != EINPROGRESS) {
				fl_message("Set channel failed: %f: %s\n",
					G.carray[n].ci_freq, strerror(rval));
			}
		} else {
#else
		if ( (rval = set_channel_internal(G.carray[n].ci_freq)) == 0) {
#endif
			G.cchoice->value(n);
			G.chan_lastidx = n;
			snprintf(chan_dpy_buf, 32, 
				"%03d: %.2f MHz", n + 1, G.carray[n].ci_freq);
			G.cdpybox->label(chan_dpy_buf);
		}
	}
}


void
cntrl_chan_prev()
{
	int i;
	if (G.carray.size() == 0)
		return;

	i = G.chan_lastidx - 1;
	if (i < 0) {
		i = G.carray.size() - 1;
	} 
	cntrl_channel_set(i);
}


void
cntrl_chan_next()
{
	int i;
	if (G.carray.size() == 0)
		return;

	i = G.chan_lastidx + 1;
	if (i >= G.carray.size()) {
		i = 0;
	} 
	cntrl_channel_set(i);
}


void
cntrl_chan_roller(Fl_Scrollbar *r)
{
	if (r->value() < 1) {		/* Scrollbar Range 0 - 999 */
		cntrl_chan_next();
	} else {
		cntrl_chan_prev();
	}
}


void
cntrl_chan_dpyinit(Fl_Box *b)
{
	current_channel = &G.chan_lastidx;
	G.cdpybox = b;	
}


void
cntrl_saa_setfd(int fd)
{
	G.saa_fd = fd;
}


void
cntrl_audio_mute()
{
	G.vol_muteval = G.vol_lastval;
	cntrl_set_volume(0, 0);
}


void
cntrl_audio_demute()
{
	cntrl_set_volume(0, G.vol_muteval);
}


void
cntrl_vol_dpyinit(Fl_Box *b)
{
	G.vdpybox = b;	
}


void
cntrl_bright_dpyinit(Fl_Box *b)
{
	G.brightdpybox = b;	
}

void
cntrl_contrast_dpyinit(Fl_Box *b)
{
	G.contdpybox = b;	
}

void
cntrl_saturation_dpyinit(Fl_Box *b)
{
	G.satdpybox = b;	
}

void
cntrl_hue_dpyinit(Fl_Box *b)
{
	G.huedpybox = b;	
}

void
cntrl_set_brightness(Fl_Scrollbar *w, int value)
{
	static char bright_dpy_buf[16];
	int v;

	v = videoopt_get_brightness();
	if (v == G.bright_lastval) {
		videoopt_set_brightness(value);
	} else {
		value = v;
		if (w)
			w->value(value, 0, 0, 255);
	}
	G.bright_lastval = value;
	snprintf(bright_dpy_buf, 32, "%3d", G.bright_lastval & 0xff);
	if (G.brightdpybox) {
		G.brightdpybox->label(bright_dpy_buf);
	}
}


void
cntrl_set_contrast(Fl_Scrollbar *w, int value)
{
	static char cont_dpy_buf[16];
	int v;

	v = videoopt_get_contrast();
	if (v == G.cont_lastval) {
		videoopt_set_contrast(value);
	} else {
		value = v;
		if (w)
			w->value(value, 0, 0, 255);
	}
	G.cont_lastval = value;
	snprintf(cont_dpy_buf, 32, "%3d", G.cont_lastval & 0xff);
	if (G.contdpybox) {
		G.contdpybox->label(cont_dpy_buf);
	}
}


void
cntrl_set_saturation(Fl_Scrollbar *w, int value)
{
	static char sat_dpy_buf[16];
	int v;

	v = videoopt_get_saturation();
	if (v == G.sat_lastval) {
		videoopt_set_saturation(value);
	} else {
		value = v;
		if (w)
			w->value(value, 0, 0, 255);
	}
	G.sat_lastval = value;
	snprintf(sat_dpy_buf, 32, "%3d", G.sat_lastval & 0xff);
	if (G.satdpybox) {
		G.satdpybox->label(sat_dpy_buf);
	}
}


void
cntrl_set_hue(Fl_Scrollbar *w, int value)
{
	static char hue_dpy_buf[16];
	int v;

	v = videoopt_get_hue();
	if (v == G.hue_lastval) {
		videoopt_set_hue(value);
	} else {
		value = v;
		if (w)
			w->value(value, 0, 0, 255);
	}
	G.hue_lastval = value;
	snprintf(hue_dpy_buf, 32, "%3d", G.hue_lastval & 0xff);
	if (G.huedpybox) {
		G.huedpybox->label(hue_dpy_buf);
	}
}


static void
set_std(enum VSTD std) 
{
	int w, h;

	stop_video(gvi, gxi);
	videoopt_set_std(std, &w, &h);
	resize_video_win(w, h);
	start_video(gvi, gxi);
}


void 
cntrl_std_choicecb(Fl_Widget *w, void *priv)
{
	Fl_Choice *c = (Fl_Choice*) w;

	set_std((enum VSTD)c->value());
}


void 
cntrl_src_choicecb(Fl_Widget *w, void *priv)
{
	Fl_Choice *c = (Fl_Choice*) w;

	videoopt_set_src((enum VSRC)c->value());
}


void 
cntrl_audio_choicecb(Fl_Widget *w, void *priv)
{
	Fl_Choice *c = (Fl_Choice*) w;
	enum OCS ocs;

	switch (c->value()) {
	case 0:
		ocs = OCS_DACLEFT_DACRIGHT;
		break;
	case 1:
		ocs = OCS_INLEFT1_INRIGHT1;
		break;
	case 2:
		ocs = OCS_INLEFT2_INRIGHT2;
		break;
	case 3:
		ocs = OCS_INLEFT1_INLEFT1;
		break;
	case 4:
		ocs = OCS_INLEFT2_INLEFT2;
		break;
	case 5:
		ocs = OCS_INRIGHT1_INRIGHT1;
		break;
	case 6:
		ocs = OCS_INRIGHT2_INRIGHT2;
		break;
	default:
		return;
	}
	set_audio_source(ocs);
}


void
cntrl_tune_dpyinit(Fl_Progress *p)
{
	G.tuneprog = p;	
}


void 
cntrl_tunechannels()
{
	vector<struct cinfo> narray;
	int i, j;
	char buf[32];

	tune_channels(G.tuneprog, &narray);
	if (yyin) {
		fpurge(yyin);
		ftruncate(fileno(yyin), 0);
		rewind(yyin);
		fseek(yyin, TVVRC_OFFSET, SEEK_SET);
		fprintf(yyin, "<CHANNELS>\n");
	}
		
	for (i = 0; i < narray.size(); ++i) {
		for (j = 0; j < G.carray.size(); ++j) {
			if (narray[i].ci_freq == G.carray[j].ci_freq) {
				if (G.carray[j].ci_name_flag == 0) {
				    strncpy(narray[i].ci_name, 
					G.carray[j].ci_name, MAX_CHANSTR);
				    narray[i].ci_name_flag = 0;
				} else {
				    snprintf(narray[i].ci_name, MAX_CHANSTR, 
					"CHANNEL %d", i + 1);
				    narray[i].ci_name_flag = 1;
				}
				break;
			}
		}
		if (j == G.carray.size()) {
			snprintf(narray[i].ci_name, MAX_CHANSTR, 
					"CHANNEL %d", i + 1);
			narray[i].ci_name_flag = 1;

		}
		if (yyin) {
			fprintf(yyin, "%.2f", narray[i].ci_freq);
			if (narray[i].ci_name_flag == 0) {
				fprintf(yyin, " %s", narray[i].ci_name);
			}
			fprintf(yyin, "\n");
		}
	}
	if (yyin)
		fflush(yyin);
	G.carray = narray;
	fl_message("%d Channels tuned.", G.carray.size());
	cntrl_channel_choiceinit(G.cchoice);
	cntrl_channel_set(0);
}


void
cntrl_src_init(Fl_Choice *c)
{
	c->value(videoopt_get_src());
}


void
cntrl_std_init(Fl_Choice *c)
{
	c->value(videoopt_get_std());
}


void
cntrl_audio_init(Fl_Choice *c)
{
	enum OCS ocs;

	if (*adev_fd != -1)
		if (ioctl(*adev_fd,  GETAUDIOMIXER, &ocs) != 0) {
	                warn("Audio get source, Ioctl error");
			return;
		}

	switch(ocs) {
	case OCS_DACLEFT_DACRIGHT:
		c->value(0);
		break;
	case OCS_INLEFT1_INRIGHT1:
		c->value(1);
		break;
	case OCS_INLEFT2_INRIGHT2:
		c->value(2);
		break;
	case OCS_INLEFT1_INLEFT1:
		c->value(3);
		break;
	case OCS_INLEFT2_INLEFT2:
		c->value(4);
		break;
	case OCS_INRIGHT1_INRIGHT1:
		c->value(5);
		break;
	case OCS_INRIGHT2_INRIGHT2:
		c->value(6);
		break;
	}
}


void
cntrl_start_ifaudio()
{
	audioopt_start_ifaudio(*adev_fd);
}
