extern "C" {
#include <sys/param.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <err.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "/sys/dev/iicbus/iic.h"

#include <saa713x_ioctl.h>

#include "../../support/tuner_ctrl.h"
}

#include <FL/Fl_Progress.H>
#include <FL/fl_ask.H>
#include <FL/Fl.H>

#include "tvv.h"
#include "cntrl.h"
#include "set_channel.h"
#include "tune_channels.h"

using namespace std;


void
tune_channels(Fl_Progress *p, vector<struct cinfo> *array)
{
	struct cinfo ci;
	struct iiccmd cmd;
	float freq, ofreq;
	int i, c;
	uint8_t rv;

	if (iic_fd == -1)
		return;

	if (iic_fd == -2) {
		iic_fd = open(IIC_PATH, O_RDWR, 0);
		if (iic_fd == -1) {
			fl_message("Error opening I2C: %s: %s", 
					IIC_PATH, strerror(errno));
			return;
		} 
	}

	bzero(&ci, sizeof(struct cinfo));
	stop_video(gvi, gxi);
	p->minimum(LOBAND_MINFREQ);
	p->maximum(HIBAND_MAXFREQ);
	p->value(LOBAND_MINFREQ);
	p->show();
	p->redraw();
	Fl::wait(0);

	if ( (i = ioctl(iic_fd, I2CRSTCARD, &cmd) != 0)) {
		fl_message("Error reseting I2C bus: %s", strerror(errno));
		return;
	}

	ofreq = 0;
	c = 0;
	freq = SCAN_START;
	for (freq = SCAN_START; freq <= SCAN_END; freq += SCAN_FSTEP) {
		c = 0;
redo:
		rv = 0;
		if ( (i = set_channel(iic_fd, freq, ofreq, &rv)) != 0) {
			if (i != EINVAL) {
			  if (++c >= 3) {
				fl_message("Error changing to frequency: "
					   "%4.2f: %s", 
					   freq, strerror(errno));
				return;
			  }
			  ioctl(iic_fd, I2CRSTCARD, &cmd);
			  goto redo;
			}
		}
		ofreq = freq;
		if ((rv & SCAN_LOCKVAL) == SCAN_LOCKVAL) {
			ci.ci_freq = freq;
			array->push_back(ci);
			freq += SCAN_SEPARATION - SCAN_FSTEP;
		}
		p->value(freq);
		p->redraw();
		Fl::wait(0);
	}

	p->hide();
	start_video(gvi, gxi);
}
