extern "C" {
#include <sys/param.h>
#include <sys/mman.h>
#include <sys/shm.h>
#include <fcntl.h>
#include <err.h>
#include <errno.h>
#include <unistd.h>
#include <sysexits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>

#include <X11/extensions/XShm.h>
#include <X11/extensions/Xvlib.h>
#include <X11/extensions/xf86dga.h>

#include <saa713x_ioctl.h>

#include "tvv_util.h"
#include "tvv.h"

#if DEBUG
#define DMSG(...)	printf(__VA_ARGS__)	
#else
#define DMSG(...)
#endif

extern int yylex(void);
extern FILE* yyin;
}

#include <FL/x.h>
#include "gui.h"
#include "cntrl.h"

#define TVV_SHMKEY	0x00001001

#define FOURCC_YV12	0x32315659
#define FOURCC_YUY2	0x32595559
#define FOURCC_RGB	0x3

#define	CLIPLIST_GROWSZ	128

int XV_IMG_WIDTH = 720;
int XV_IMG_HEIGHT = 576;


struct vinfo {
	struct saa_video_spec	vi_svs;
	int 			vi_vfmt;
	enum SWAPBYTES 		vi_vfmt_swapbytes;
	int			vi_dfd;
	struct saa_status 	*vi_dsts;
	char			*vi_dname;
	uint8_t			vi_flag_on;
};

struct ainfo {
	int			ai_ocs_initflag;
	enum OCS		ai_ocs;
	int			ai_dfd;
	struct saa_status 	*ai_dsts;
	char			*ai_dname;
};

struct dgainfo {
	int			dga_flags;
	int			dga_ppl;
	int			dga_bypp;
	void			*dga_base;
	char 			dga_mode;

	XtWorkProcId		dga_wproc_id;
};

struct xvinfo {
	XShmSegmentInfo 	xv_xshminfo;
	GC			xv_gc;

	int			xv_shmid;
	XvPortID		xv_port;
	XvImage			*xv_img;

	int			xv_img_height;
	int			xv_framesz;
	uint32_t		xv_fourcc;

 	uint32_t		xv_mbuf_ctr;
 
 	char			xv_multibuf;
	char			xv_nodshm;
	char			xv_mode;
	uint8_t			*xv_shmaddr;
	uint8_t			*xv_mmapaddr;
};

struct CI_A {
	Window		win;
	uint32_t	x, y, w, h;
};

struct clipinfo {
	struct CI_A	ci_a;
	int32_t		ci_skip;
	uint32_t	ci_ox, ci_oy, ci_ow, ci_oh;
};
	
struct xinfo {
	XtAppContext	 	xi_app;
	Display			*xi_display;
	Widget 			xi_win;

	short			xi_scr_width;
	short			xi_scr_height;
	short			xi_win_width;
	short			xi_win_height;

	short			xi_win_xpos;
	short			xi_win_ypos;
	char			xi_win_resetpending;
	char			xi_win_mapped;

	struct dgainfo		*xi_dgainfo;
	struct xvinfo		*xi_xvinfo;

	Window			xi_wm_parent;
	Fl_Window		*xi_fl_win;

	uint32_t		xi_clist_sz;
	uint32_t		xi_clist_count;
	struct clipinfo		*xi_clist;
	uint32_t		xi_set_clist_count;
	struct clipinfo		xi_set_clist[SAA_MAX_CLIPS];
};


static void
alloc_defaults(struct vinfo **vi, struct ainfo **ai, struct xinfo **xi)
{
	if ( (*vi = (struct vinfo*) calloc(1, sizeof(struct vinfo))) == NULL ||
	     (*ai = (struct ainfo*) calloc(1, sizeof(struct ainfo))) == NULL ||
	     (*xi = (struct xinfo*) calloc(1, sizeof(struct xinfo))) == NULL)
		err(ENOMEM, "alloc_defaults");

	if (((*xi)->xi_dgainfo = (struct dgainfo*) 
				calloc(1, sizeof(struct dgainfo))) == NULL ||
	     ((*xi)->xi_xvinfo =  (struct xvinfo*)
				calloc(1, sizeof(struct xvinfo))) == NULL)
		err(ENOMEM, "alloc_defaults");

	/* VI Defaults */
	(*vi)->vi_vfmt = VFMT_YUV422;
	(*vi)->vi_vfmt_swapbytes = SWAPBYTES_2143;
	(*vi)->vi_dname = "/dev/saa0";

	/* AI Defaults */
	(*ai)->ai_ocs = OCS_INLEFT2_INRIGHT2;
	(*ai)->ai_dname = "/dev/sau0";

	/* X Defaults */
	(*xi)->xi_win_width = 720;
	(*xi)->xi_win_height = 576;
	(*xi)->xi_win_resetpending = 1;
	

	/* DGA Defaults */
	(*xi)->xi_dgainfo->dga_mode = 0;

	/* Xv Defaults */
	(*xi)->xi_xvinfo->xv_mode = 'w';
	(*xi)->xi_xvinfo->xv_fourcc = FOURCC_YUY2;
	(*xi)->xi_xvinfo->xv_nodshm = 1;
 	(*xi)->xi_xvinfo->xv_multibuf = 0;
}


static __inline void
xv_setmodeinfo(uint32_t fourcc, struct vinfo *vi, struct xinfo *xi)
{
	switch (fourcc) {
		case FOURCC_YUY2:
			xi->xi_xvinfo->xv_fourcc = FOURCC_YUY2;
			vi->vi_vfmt = VFMT_YUV422;
			vi->vi_vfmt_swapbytes = SWAPBYTES_2143;
			break;
		case FOURCC_YV12:
			xi->xi_xvinfo->xv_fourcc = FOURCC_YV12;
			vi->vi_vfmt = VFMT_PLANAR_YUV422;
			vi->vi_vfmt_swapbytes = SWAPBYTES_NONE;
			break;
		case FOURCC_RGB:
			xi->xi_xvinfo->xv_fourcc = FOURCC_RGB;
			vi->vi_vfmt = VFMT_RGBa888;
			vi->vi_vfmt_swapbytes = SWAPBYTES_NONE;
			break;
		default:
			errx(ENOTSUP, 
				"Xv fourcc %d not supported/implemented",
				fourcc);
		}
}


/* Sound option DAC -> SndCard via /dev/pcm */

const char *optstr = "Xf:D:V:F:a:A:M";
const char *usage = 
	"-X			: Enable DSHM\n"
	"-M			: Enable MULTIBUF\n"
	"-f <device>		: Capture device, /dev/saa0 (default)\n"
    	"-D <mode>		: Playback using DGA (default), mode is:\n"
	"			    w - Weave\n" 
	"			    b - Bob\n"
	"			    o - Drop field 1 (odd lines)\n"
	"			    e - Drop field 2 (even lines)\n"
    	"-V <mode>		: Playback using Xv, mode is:\n"
	"			    w - Weave (DEFAULT)\n" 
	"			    r - RT-Weave\n" 
	"			    b - Bob\n"
	"			    o - Drop field 2 (odd lines)\n"
	"			    e - Drop field 1 (even lines)\n"
	"-F <format>		: Xv Playback format, format is:\n"
	"			    u - YUY2\n"
	"			    y - YV12\n"
	"			    r - RGB\n"
	"-a <mode>		: Audio Lineout, output selection:\n"
	"			    d - DAC\n"
	"			    1 - Linein 1, stereo\n"
	"			    2 - Linein 2, stereo (default)\n"
	"			    3 - Linein 1, left mono\n"
	"			    4 - Linein 1, right mono\n"
	"			    5 - Linein 2, left mono\n"
	"			    6 - Linein 2, right mono\n"
	"-A <device>		: Audio device,  /dev/sau0 (default)\n";


static void
parse_args(int argc, char **argv, 
		struct vinfo *vi, struct ainfo *ai, struct xinfo *xi)
{
	int ch;

	while ( (ch = getopt(argc, argv, optstr)) != -1) {
		switch(ch) {
		case 'M':
			xi->xi_xvinfo->xv_multibuf = 1;
			break;
		case 'X':
			xi->xi_xvinfo->xv_nodshm = 0;
			break;
		case 'f':
			vi->vi_dname = optarg;
			break;
		case 'D':
			switch (optarg[0]) {
			case 'w':
			case 'b':
			case 'o':
			case 'e':
				xi->xi_dgainfo->dga_mode = optarg[0];
				break;
			default:
				errx(ENOTSUP, 
					"Dga mode %c not supported/implemented"
					"\nUsage:\n%s",
					optarg[0], usage);
			}
			break;
		case 'V':
			switch (optarg[0]) {
			case 'w':
			case 'r':
			case 'b':
			case 'o':
			case 'e':
				xi->xi_dgainfo->dga_mode = 0;
				xi->xi_xvinfo->xv_mode = optarg[0];
				break;
			default:
				errx(ENOTSUP, 
					"Xv mode %c not supported\nUsage:\n%s",
					optarg[0], usage);
			}
			break;
		case 'F':
			switch (optarg[0]) {
			case 'u':
				xv_setmodeinfo(FOURCC_YUY2, vi, xi);
				break;
			case 'y':
				xv_setmodeinfo(FOURCC_YV12, vi, xi);
				break;
			case 'r':
				xv_setmodeinfo(FOURCC_RGB, vi, xi);
				break;
			default:
				errx(ENOTSUP, 
					"Xv format %c not supported/implemented"
					"\nUsage:\n%s",
					optarg[0], usage);
			}
			break;
		case 'a':
			switch(optarg[0]) {
 			case 'd':
				ai->ai_ocs = OCS_DACLEFT_DACRIGHT;
				break;
			case '1':
				ai->ai_ocs = OCS_INLEFT1_INRIGHT1;
				break;
			case '2':
				ai->ai_ocs = OCS_INLEFT2_INRIGHT2;
				break;
			case '3':
				ai->ai_ocs = OCS_INLEFT1_INLEFT1;
				break;
			case '4':
				ai->ai_ocs = OCS_INLEFT2_INLEFT2;
				break;
			case '5':
				ai->ai_ocs = OCS_INRIGHT1_INRIGHT1;
				break;
			case '6':
				ai->ai_ocs = OCS_INRIGHT2_INRIGHT2;
				break;
			default:
				errx(ENOTSUP, 
					"Invalid audio Lineout, "
					" mode %c not supported"
					"\nUsage:\n%s",
					optarg[0], usage);
			}
			ai->ai_ocs_initflag = 1;
			break;
		case 'A':
			ai->ai_dname = optarg;
			break;
		default:
			errx(ENOTSUP, "Invalid argument %c\nUsage:\n%s",
				ch, usage);
			break;
		}
	}
}

struct privdata {
	struct vinfo *vi;
	struct ainfo *ai;
	struct xinfo *xi;
};

static void* fltk_thread(void *arg);
static void event_handler(Widget w, XtPointer priv, XEvent *e, Boolean *cont);
static void audio_init(struct ainfo *ai);
static void dga_init(struct vinfo *vi, struct xinfo *xi);
static void xv_init(struct vinfo *vi, struct xinfo *xi);
static void status_notify(int sig);
static void xv_frame_notify(int sig);
static void xv_multibuf_notify(int sig);
static void reset_viewport(struct vinfo *vi, struct xinfo *xi);

static __inline void cliplist_clear(struct xinfo *xi);
static __inline void cliplist_add(struct xinfo *xi, struct clipinfo *ci);
static int clipinfo_compare(const void *a, const void *b);
static void cliplist_sort(struct xinfo *xi);
static void cliplist_merge(struct xinfo *xi);
static void clipping_reset(struct vinfo *vi, struct xinfo *xi);
static void clipping_set_cliplist(struct vinfo *vi, struct xinfo *xi);
static void clipping_reconfigure(struct vinfo *vi, struct xinfo *xi, 
					Window owin, int *rval);

static void generate_saa_cliplist(struct xinfo *xi, 
					struct saa_video_cliplist *svcl);

static int xgetwinattr_errorhandler(Display *d, XErrorEvent *e);

/* Required due to signal handler processing */
struct vinfo *gvi;
struct ainfo *gai;
struct xinfo *gxi;

int *adev_fd;

static int
uevent_handler(int fe)
{
	XWindowAttributes wattr;
	XErrorHandler eh;
	int i;
	XEvent *e = (XEvent*) fl_xevent;

	XLockDisplay(gxi->xi_display);
	if (gxi->xi_dgainfo->dga_mode) {
		if (e->xany.window == XtWindow(gxi->xi_win)) {
			if (e->type == ButtonRelease) {
				switch(e->xbutton.button) {
				case 1:
				case 4:
					cntrl_chan_prev();
					break;
				case 3:
				case 5:
					cntrl_chan_next();
					break;
				case 2:
					if (gxi->xi_fl_win->visible())
						gxi->xi_fl_win->iconize();	
					else
						gxi->xi_fl_win->show();	
					break;
				}
			} else
				XtDispatchEvent(e);
			XUnlockDisplay(gxi->xi_display);
			return 1;
		} else if ((e->type == ConfigureNotify || 
					e->type == CreateNotify) && 
			   gxi->xi_wm_parent != 0) {
			/* reconfig onyly if window is in clist & 
			 * pos./dim. change or 
			 * window's rect is within our video win's rect
			 */

			/* Clist, Position & Dimension check */
			for (i = 0; i < gxi->xi_set_clist_count; ++i) {
				if (e->xconfigure.window ==
						 gxi->xi_set_clist[i].ci_a.win) {
				    wattr.width = 0;
				    eh = XSetErrorHandler(
						xgetwinattr_errorhandler);
				    XGetWindowAttributes(gxi->xi_display,
							 e->xconfigure.window,
							 &wattr);
				    XSetErrorHandler(eh);
				    if (wattr.width == 0)
					continue;

				    if (!(wattr.map_state & IsViewable) ||
					wattr.x != gxi->xi_set_clist[i].ci_ox ||
					wattr.y != gxi->xi_set_clist[i].ci_oy ||
					wattr.width != 
						gxi->xi_set_clist[i].ci_ow ||
					wattr.height != 
						gxi->xi_set_clist[i].ci_oh
				    ) {
					    DMSG("Pos -> clip_reconfig\n");
					    clipping_reconfigure(gvi, gxi, 0,0);
					    if (gxi->xi_win_mapped == 1 && 
						gvi->vi_flag_on == 0) {
							start_video(gvi, gxi);
					    }
				    }
				    XUnlockDisplay(gxi->xi_display);
				    return 1;
				}
			}

			/*RectInRegion check */
			clipping_reconfigure(gvi, gxi, e->xconfigure.window,&i);
			if (i != RectangleOut) {
				DMSG("InRect -> clip_reconfig\n");
				clipping_reconfigure(gvi, gxi, 0, 0);
				if (gxi->xi_win_mapped == 1 && 
					gvi->vi_flag_on == 0) {
						start_video(gvi, gxi);
			        }
			}
			XUnlockDisplay(gxi->xi_display);
			return 1;
	    	}
	} else {
		/* Xv Mode */
		if (e->xany.window == XtWindow(gxi->xi_win)) {
			if (e->type == ButtonRelease) {
				switch(e->xbutton.button) {
				case 1:
				case 4:
					cntrl_chan_prev();
					break;
				case 3:
				case 5:
					cntrl_chan_next();
					break;
				case 2:
					if (gxi->xi_fl_win->visible())
						gxi->xi_fl_win->iconize();	
					else
						gxi->xi_fl_win->show();	
					break;
				}
			} else 
				XtDispatchEvent(e);
			XUnlockDisplay(gxi->xi_display);
			return 1;
		}
	}
	XUnlockDisplay(gxi->xi_display);
	return 0;
}

int
main(int argc, char **argv)
{
	XSetWindowAttributes cattr;
	struct privdata p;
	struct vinfo *vi;
	struct ainfo *ai;
	struct xinfo *xi;
	Screen *scr;
	pthread_t fthread;
	char *fname;
	int fd, saved_channel;
	char buf[4];

	alloc_defaults(&p.vi, &p.ai, &p.xi);
	gvi = vi = p.vi; gai = ai = p.ai; gxi = xi = p.xi;
	XInitThreads();
	if ( (vi->vi_dfd = open(vi->vi_dname, O_RDWR)) == -1)
		err(errno, "error opening %s", vi->vi_dname);
	videoopt_init(vi->vi_dfd, &XV_IMG_WIDTH, &XV_IMG_HEIGHT);
	xi->xi_win_width = XV_IMG_WIDTH;
	xi->xi_win_height = XV_IMG_HEIGHT;
	
#if 0
	xi->xi_win = XtVaOpenApplication(&xi->xi_app, "TVV",
					NULL, 0,
					&argc, argv,
					NULL,
					applicationShellWidgetClass,
					NULL);
	xi->xi_display = XtDisplay(xi->xi_win);
#else
	XtToolkitInitialize();
	xi->xi_app = XtCreateApplicationContext();
	xi->xi_display = XtOpenDisplay(xi->xi_app, NULL, argv[0], "TVV",
				0, 0,
				&argc, argv);
	fl_open_display(xi->xi_display);
	XLockDisplay(gxi->xi_display);
	xi->xi_win = XtVaAppCreateShell("tvv", "TVV",
					applicationShellWidgetClass,
					xi->xi_display,	
					NULL);

#endif

	scr = DefaultScreenOfDisplay(xi->xi_display);
        xi->xi_scr_width = WidthOfScreen(scr);
        xi->xi_scr_height = HeightOfScreen(scr);

	/* Needs to be done after XtOpenDisplay, so that
	 * X argumets are processed correctly
	 */
	parse_args(argc, argv, vi, ai, xi);
	
	fname = (char*) malloc(128);
	strncpy(fname, getenv("HOME"), 128);
	strncat(fname, "/.tvvrc", 128);
	if ( (yyin = fopen(fname, "r+")) != NULL || 
	     (yyin = fopen(fname, "w+")) != NULL) {
		fseek(yyin, TVVRC_OFFSET, SEEK_SET);
		yylex();
		fseek(yyin, 0, SEEK_SET);
		buf[0] = buf[1] = buf[2] = buf[3] = 0;
		fread(buf, 4, 1, yyin);
		saved_channel = atoi(buf);
	}
	free(fname);

	cntrl_saa_setfd(vi->vi_dfd);

	audio_init(ai);

	if (xi->xi_dgainfo->dga_mode) {
		/* SAA Clipping Parameters */
		vi->vi_svs.svs_clipmode.svcm_mode = CLIPMODE_RECT;
		vi->vi_svs.svs_clipmode.svcm_flag_clinv = 0;
		vi->vi_svs.svs_clipmode.svcm_flag_intlcd = 1;
		if (xi->xi_dgainfo->dga_mode == 'o' || 
		    xi->xi_dgainfo->dga_mode == 'e')
			vi->vi_svs.svs_clipmode.svcm_flag_intlcd = 0;
		dga_init(vi, xi);
	}

	XtAddEventHandler(xi->xi_win,
				StructureNotifyMask | VisibilityChangeMask |
				 ExposureMask | ButtonReleaseMask,
				FALSE,
				event_handler, 
				&p);

	XtVaSetValues(xi->xi_win, 
			XtNwidth, xi->xi_win_width,
			XtNheight, xi->xi_win_height,
			NULL);

	XtRealizeWidget(xi->xi_win);

	cattr.background_pixel = BlackPixel(xi->xi_display, 0);
	XChangeWindowAttributes(xi->xi_display,
				XtWindow(xi->xi_win),
				CWBackPixel,
				&cattr);

	if (!xi->xi_dgainfo->dga_mode)
		xv_init(vi, xi);

	XSelectInput(xi->xi_display,
			DefaultRootWindow(xi->xi_display),
			SubstructureNotifyMask);

	Fl_Window *w = xi->xi_fl_win = make_window();
	w->label("tvv control");
	w->end();
	w->show();
	Fl::add_handler(uevent_handler);
	cntrl_channel_set(saved_channel);
	XUnlockDisplay(gxi->xi_display);

	pthread_create(&fthread, NULL, fltk_thread, NULL);
	while(1)
		pause();
	return 0;
}

static void*
fltk_thread(void *arg)
{
	return (void*)Fl::run();
}

static __inline void
cliplist_clear(struct xinfo *xi)
{
	xi->xi_clist_count = 0;	
}

static __inline void
cliplist_add(struct xinfo *xi, struct clipinfo *ci)
{
	struct clipinfo *nci;
	int i;

	if (xi->xi_clist_count >= xi->xi_clist_sz) {
		nci = (struct clipinfo*)malloc(sizeof(struct clipinfo) * 
				(xi->xi_clist_sz + CLIPLIST_GROWSZ));
		if (!nci)
			errx(ENOMEM, "error allocating cliplist memory");
		memcpy(nci, xi->xi_clist, 
			sizeof(struct clipinfo) * xi->xi_clist_count);
		free(xi->xi_clist);
		xi->xi_clist = nci;
		xi->xi_clist_sz += CLIPLIST_GROWSZ;
	}
	
	xi->xi_clist[xi->xi_clist_count] = *ci;
	++xi->xi_clist_count;

	DMSG("InClipAdd (0x%x): x: %d y: %d w: %d h: %d\n", 
		 ci->ci_a.win, ci->ci_a.x, ci->ci_a.y, ci->ci_a.w, ci->ci_a.h);
}

static int
clipinfo_compare(const void *a, const void *b)
{
	struct clipinfo *cia, *cib;
	uint32_t i, j;
	
	cia = (struct clipinfo*) a;
	cib = (struct clipinfo*) b;

	i = cia->ci_skip == -1 ? 0 : 1;
	j = cib->ci_skip == -1 ? 0 : 1;

	if (i != j) {
		if (i)
			return 1;
		else
			return -1;
	}

	/* Invariant, i == j */
	if (i == 1)
		return 0;

	i = cia->ci_a.x;
	j = cib->ci_a.x;
	if (i != j) {
		if (i > j)
			return 1;
		else
			return -1;
	}

	i = cia->ci_a.y;
	j = cib->ci_a.y;
	if (i != j) {
		if (i > j)
			return 1;
		else
			return -1;
	}

	i = cia->ci_a.w;
	j = cib->ci_a.w;
	if (i != j) {
		if (i > j)
			return 1;
		else
			return -1;
	}

	i = cia->ci_a.h;
	j = cib->ci_a.h;
	if (i != j) {
		if (i > j)
			return 1;
		else
			return -1;
	}

	/* Regions are identical, should not occur as identical regions
	 *  are merged by clipist_merge
	 */
	return 0;
}

static void
cliplist_sort(struct xinfo *xi)
{
	uint32_t i;

	if (xi->xi_clist_count <= 1)
		return;

	qsort(xi->xi_clist, xi->xi_clist_count, sizeof(struct clipinfo),
		clipinfo_compare);

	for (i = 0; i < xi->xi_clist_count; ++i)
		if (xi->xi_clist[i].ci_skip != -1)
			break;

	xi->xi_clist_count = i;
}

static void
cliplist_merge(struct xinfo *xi)
{
	XPoint points[4];
	Region region;
	uint32_t i, j;

	for (i = 0; i < xi->xi_clist_count; ++i) {

		if (xi->xi_clist[i].ci_skip != -1)
			continue;

		points[0].x = xi->xi_clist[i].ci_a.x;
		points[0].y = xi->xi_clist[i].ci_a.y;
		points[1].x = xi->xi_clist[i].ci_a.x;
		points[1].y = xi->xi_clist[i].ci_a.y + xi->xi_clist[i].ci_a.h;
		points[2].x = xi->xi_clist[i].ci_a.x + xi->xi_clist[i].ci_a.w;
		points[2].y = xi->xi_clist[i].ci_a.y + xi->xi_clist[i].ci_a.h;
		points[3].x = xi->xi_clist[i].ci_a.x + xi->xi_clist[i].ci_a.w;
		points[3].y = xi->xi_clist[i].ci_a.y;
		region = XPolygonRegion(points, 4, WindingRule);

		for (j = 0; j < xi->xi_clist_count; ++j) {
			if (xi->xi_clist[j].ci_skip == -1 && j != i &&
			    (XRectInRegion(region, 
				xi->xi_clist[j].ci_a.x,
				xi->xi_clist[j].ci_a.y,
				xi->xi_clist[j].ci_a.w,
				xi->xi_clist[j].ci_a.h) == RectangleIn)) 
			{
				xi->xi_clist[j].ci_skip = i;
				DMSG("Merging Region %d into %d\n", j, i);
			}
		}
		XDestroyRegion(region);
	}
}

static void
clipping_reset(struct vinfo *vi, struct xinfo *xi)
{
	struct saa_video_clipinfo svci;

	if (xi->xi_set_clist_count) {
		DMSG("cliplist reset\n\n");
		xi->xi_set_clist_count = 0;
		if (vi->vi_flag_on) {
			svci.svci_clipmode = vi->vi_svs.svs_clipmode;
			generate_saa_cliplist(xi, &svci.svci_cliplist);
			if (ioctl(vi->vi_dfd, SETCLIPINFO, &svci) != 0)
				warn("Error setting SAA clipping information");
		}
	}
}

static Boolean
refresh_desktop(XtPointer data)
{
	struct xinfo *xi;

	/* XXX Fix refresh code, avoid call system(...) */
#if 0
	XSetWindowAttributes attr;
	Display *d;
	Window w;
        unsigned long valuemask;

	xi = (struct xinfo *)data;
	d = xi->xi_display;

	DMSG("Refreshing Desktop\n");
	valuemask = (CWBackPixel | CWBackingStore | CWSaveUnder);
	attr.background_pixel = BlackPixel(xi->xi_display, 0);
	attr.backing_store = NotUseful;
	attr.save_under = False;
	w = XCreateWindow(d,
				DefaultRootWindow(d), 
				0, 0,
				xi->xi_scr_width,
				xi->xi_scr_height,
				0,
				CopyFromParent, CopyFromParent,
				CopyFromParent, valuemask,
				&attr);
	XMapWindow(d, w);
	XDestroyWindow(d, w);
	XFlush(d);

	if (xi->xi_dgainfo->dga_wproc_id != 0) {
		XtRemoveWorkProc(xi->xi_dgainfo->dga_wproc_id);
		xi->xi_dgainfo->dga_wproc_id = 0;
	}
#else
	xi = (struct xinfo *)data;
	DMSG("Refreshing Desktop\n");
	usleep(1000);
	system("xrefresh");
	if (xi->xi_dgainfo->dga_wproc_id != 0) {
		XtRemoveWorkProc(xi->xi_dgainfo->dga_wproc_id);
		xi->xi_dgainfo->dga_wproc_id = 0;
	}
#endif
	return TRUE;
}

static void
clipping_set_cliplist(struct vinfo *vi, struct xinfo *xi)
{
	struct saa_video_clipinfo svci;
	uint32_t count, i;

	count = xi->xi_clist_count > SAA_MAX_CLIPS ? 
					SAA_MAX_CLIPS : xi->xi_clist_count;

	/* Check if new clist is same as already set list */
	if (xi->xi_set_clist_count == count) {
		/* XXX memcmp cannot be used directly due to ci_o* members 
		 * we don't want include them in the comparison
		 */
		for (i = 0; i < count; ++i) {
			if (memcmp(&xi->xi_set_clist[i].ci_a,
				   &xi->xi_clist[i].ci_a,
				   sizeof (struct CI_A)) != 0)
				break;
		}
		if (i >= count) {
			DMSG("cliplist: no change, count: %d\n\n", count);
			return;
		}
	}
	if (count == 0) {
		clipping_reset(vi, xi);	
	} else {
		DMSG("cliplist: changed, count: %d\n\n", count);
		memcpy(xi->xi_set_clist, xi->xi_clist,
				sizeof(struct clipinfo) * count);
		xi->xi_set_clist_count = count;
		if (vi->vi_flag_on) {
			svci.svci_clipmode = vi->vi_svs.svs_clipmode;
			generate_saa_cliplist(xi, &svci.svci_cliplist);
			if (ioctl(vi->vi_dfd, SETCLIPINFO, &svci) != 0)
				warn("Error setting SAA clipping information");
		}
	}

#if XXX_REFRESH_DOESNOT_WORKWITH_FLTK
	if (xi->xi_dgainfo->dga_wproc_id == 0) {
		xi->xi_dgainfo->dga_wproc_id =
			XtAppAddWorkProc(xi->xi_app, refresh_desktop, xi);
	}
#endif
}


static void
clipping_reconfigure(struct vinfo *vi, struct xinfo *xi, Window owin, int *rval)
{
	XPoint points[4];
	XWindowAttributes wattr;
	XErrorHandler eh;
	struct clipinfo cinfo;
	Window root, win, uroot, uparent, *children;
	unsigned int count, i;
	int x, y, w, h;
	Region region;


	root = DefaultRootWindow(xi->xi_display);
	win = XtWindow(xi->xi_win);

	if (xi->xi_wm_parent == 0) {
		xi->xi_wm_parent = win;
		for (;;) {
			XQueryTree(xi->xi_display, xi->xi_wm_parent, 
					&uroot, &uparent, &children, &count);
			XFree(children);
			if (uparent == root)
				break;
			xi->xi_wm_parent = uparent;
		}
	}

	if (owin == 0) {
		XQueryTree(xi->xi_display, root, &uroot, &uparent, 
				&children, &count);
		for (i = 0; i < count; ++i)
			if (children[i] == xi->xi_wm_parent)
				break;
	}

	w = xi->xi_win_width;
	h = xi->xi_win_height;
	x = xi->xi_win_xpos;
	y = xi->xi_win_ypos;

	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
	if ((x + w) > xi->xi_scr_width)
		w = xi->xi_scr_width - x;
	if ((y + h) > xi->xi_scr_height)
		h = xi->xi_scr_height - y;	

	points[0].x = x;
	points[0].y = y;
	points[1].x = x;
	points[1].y = y + h;
	points[2].x = x + w;
	points[2].y = y + h;
	points[3].x = x + w;
	points[3].y = y;
	region = XPolygonRegion(points, 4, WindingRule);

	if (owin == 0)
		cliplist_clear(xi);

	/* Handle negative positions */
	if (xi->xi_win_xpos < 0) {
		/* No need to set ci_o* members */
		cinfo.ci_a.win = win;
		cinfo.ci_a.x = 0;
		cinfo.ci_a.y = 0;
		cinfo.ci_a.w = -xi->xi_win_xpos;
		cinfo.ci_a.h = xi->xi_win_height;
		cinfo.ci_skip = -1;
		if (owin == 0)
			cliplist_add(xi, &cinfo);
	}
	if (xi->xi_win_ypos < 0) {
		/* No need to set ci_o* members */
		cinfo.ci_a.win = win;
		cinfo.ci_a.x = 0;
		cinfo.ci_a.y = 0;
		cinfo.ci_a.w = xi->xi_win_width;
		cinfo.ci_a.h = -xi->xi_win_ypos;
		cinfo.ci_skip = -1;
		if (owin == 0)
			cliplist_add(xi, &cinfo);
	}

	if (owin != 0) {
		wattr.width = 0;
		eh = XSetErrorHandler(xgetwinattr_errorhandler);
		XGetWindowAttributes(xi->xi_display, owin, &wattr);
		XSetErrorHandler(eh);
		if (wattr.width != 0 && (wattr.map_state & IsViewable)) {
			*rval = XRectInRegion(region, wattr.x, wattr.y, 
						wattr.width, wattr.height);
		} else {
			*rval = RectangleOut;
		}
		XDestroyRegion(region);
		return;
	}

	DMSG("Region Rect: x: %d, y: %d, w: %d, h: %d\n", x, y, w, h);

	for (++i; i < count; ++i) {
		wattr.width = 0;
		eh = XSetErrorHandler(xgetwinattr_errorhandler);
		XGetWindowAttributes(xi->xi_display, children[i], &wattr);
		XSetErrorHandler(eh);
		if (wattr.width == 0 || !(wattr.map_state & IsViewable))
			continue;

		/* XXX Account for window border */
		wattr.width += wattr.border_width;
		wattr.height += wattr.border_width;

		switch (XRectInRegion(region, wattr.x, wattr.y, 
					wattr.width, wattr.height)) {
		case RectangleIn:
			cinfo.ci_a.win = children[i];
			cinfo.ci_ox = wattr.x;
			cinfo.ci_oy = wattr.y;
			cinfo.ci_ow = wattr.width;
			cinfo.ci_oh = wattr.height;
			cinfo.ci_a.x = wattr.x - xi->xi_win_xpos;
			cinfo.ci_a.y = wattr.y - xi->xi_win_ypos;
			cinfo.ci_a.w = wattr.width;
			cinfo.ci_a.h = wattr.height;
			cinfo.ci_skip = -1;
			cliplist_add(xi, &cinfo);
			break;
		case RectanglePart:
			w = wattr.width;
			h = wattr.height;
			x = wattr.x < xi->xi_win_xpos ? 
						xi->xi_win_xpos : wattr.x;
			y = wattr.y < xi->xi_win_ypos ? 
						xi->xi_win_ypos : wattr.y;
			w -= x - wattr.x;
			h -= y - wattr.y;

			if ((x + w) > (xi->xi_win_xpos + xi->xi_win_width))
				w = (xi->xi_win_xpos + xi->xi_win_width) - x;

			if ((y + h) > (xi->xi_win_ypos + xi->xi_win_height))
				h = (xi->xi_win_ypos + xi->xi_win_height) - y;

			cinfo.ci_a.win = children[i];
			cinfo.ci_ox = wattr.x;
			cinfo.ci_oy = wattr.y;
			cinfo.ci_ow = wattr.width;
			cinfo.ci_oh = wattr.height;
			cinfo.ci_a.x = x - xi->xi_win_xpos;
			cinfo.ci_a.y = y - xi->xi_win_ypos;
			cinfo.ci_a.w = w;
			cinfo.ci_a.h = h;
			cinfo.ci_skip = -1;
			cliplist_add(xi, &cinfo);
			break;
		}
	}
	XDestroyRegion(region);
	XFree(children);

	cliplist_merge(xi);
	cliplist_sort(xi);
	clipping_set_cliplist(vi, xi);
}


static void
event_handler(Widget w, XtPointer priv, XEvent *e, Boolean *cont)
{
	struct privdata *gptr;
	struct vinfo *vi;
	struct ainfo *ai;
	struct xinfo *xi;
	unsigned int i;
	short x, y;

	gptr = (struct privdata*) priv;
	vi = gptr->vi; ai = gptr->ai; xi = gptr->xi;

	if (e->xany.window != XtWindow(xi->xi_win)) {
		*cont = TRUE;
		return;
	}

	XLockDisplay(gxi->xi_display);
	switch (e->type) {
	case Expose:
		if (!xi->xi_dgainfo->dga_mode)
			break;
		clipping_reconfigure(vi, xi, 0, 0);
		if (xi->xi_win_mapped == 1 && vi->vi_flag_on == 0) 
			start_video(vi, xi);
		break;
	case VisibilityNotify:
		if (!xi->xi_dgainfo->dga_mode)
			break;
		switch (e->xvisibility.state) {
		case VisibilityUnobscured:
			clipping_reset(vi, xi);
			if (xi->xi_win_mapped == 1 && vi->vi_flag_on == 0) 
				start_video(vi, xi);
			break;
		case VisibilityFullyObscured:
			if (xi->xi_dgainfo->dga_mode)
				stop_video(vi, xi);
			break;
		case VisibilityPartiallyObscured:
			clipping_reconfigure(vi, xi, 0, 0);
			if (xi->xi_win_mapped == 1 && vi->vi_flag_on == 0)
				start_video(vi, xi);
			break;
		}
		break;
	case UnmapNotify:
		xi->xi_win_mapped = 0;
		stop_video(vi, xi);
		if (!xi->xi_dgainfo->dga_mode)
			xi->xi_win_resetpending = 1;
		break;
	case MapNotify:
		xi->xi_win_mapped = 1;
		if (xi->xi_win_resetpending == 1) {
			xi->xi_win_resetpending = 0;
			reset_viewport(vi, xi);
		}
		start_video(vi, xi);
		break;
	case ConfigureNotify:
		i = 0;
		/* XXX cannot rely on xconfigure.[xy] as
		 * they become 0, 0 when win is resized
		 */
		XtVaGetValues(xi->xi_win,
				XtNx, &x,	
				XtNy, &y,	
				NULL);
		if (xi->xi_win_width != e->xconfigure.width) {
			if (!xi->xi_dgainfo->dga_mode && 
				(xi->xi_win_width >= XV_IMG_WIDTH &&
				 e->xconfigure.width >= XV_IMG_WIDTH))
				xi->xi_win_width = e->xconfigure.width;
			else {
				xi->xi_win_width = e->xconfigure.width;
				++i;
			}
		}
		if (xi->xi_win_height != e->xconfigure.height) {
			if (!xi->xi_dgainfo->dga_mode && 
				(xi->xi_win_height >= XV_IMG_HEIGHT &&
				 e->xconfigure.height >= XV_IMG_HEIGHT))
				xi->xi_win_height = e->xconfigure.height;
			else {
				xi->xi_win_height = e->xconfigure.height;
				++i;
			}
		}
		if (xi->xi_win_xpos != x) {
			xi->xi_win_xpos = x;
			if (xi->xi_dgainfo->dga_mode)
				++i;
		}
		if (xi->xi_win_ypos != y) {
			xi->xi_win_ypos = y;
			if (xi->xi_dgainfo->dga_mode)
				++i;
		}
		if (i) {
			xi->xi_win_resetpending = 1;
			if (xi->xi_win_mapped) {
				xi->xi_win_resetpending = 0;
				clipping_reconfigure(vi, xi, 0, 0);
				reset_viewport(vi, xi);
				start_video(vi, xi);
			}
			if (xi->xi_dgainfo->dga_mode &&
			    xi->xi_dgainfo->dga_wproc_id == 0)
			{
			    xi->xi_dgainfo->dga_wproc_id =
				XtAppAddWorkProc(xi->xi_app, refresh_desktop, xi);
			}
		}
		break;
	}
	*cont = False;
	XUnlockDisplay(gxi->xi_display);
}


void
set_audio_source(enum OCS ocs)
{
	if (gai->ai_dfd != -1)
		if (ioctl(gai->ai_dfd,  SETAUDIOMIXER, &ocs) != 0)
                	warn("Audio set source, Ioctl error");
}


static void
audio_init(struct ainfo *ai)
{
	adev_fd = &ai->ai_dfd;
	if ( (ai->ai_dfd = open(ai->ai_dname, O_RDWR)) == -1) {
		warn("error opening %s", ai->ai_dname);
		ai->ai_dfd = -1;
		return;
	}
	if (ai->ai_ocs_initflag != 0) {
		if (ioctl(ai->ai_dfd, SETAUDIOMIXER, &ai->ai_ocs) != 0)
                	warn("Audio set source, Ioctl error");
	}
}


static void
dga_init(struct vinfo *vi, struct xinfo *xi)
{
	struct sigaction sa;
	sigset_t ss;
	int depth, s, dummy1, dummy2;

	sa.sa_flags = 0;
	sigemptyset(&ss);
	sa.sa_mask = ss;
	sa.sa_handler = status_notify;
	if (sigaction(SIGUSR2, &sa, NULL) != 0)
		err(errno, "error setting up status signal handler (USR2)");
	if ( (vi->vi_dsts = 
		(struct saa_status*) mmap(0, SAA_MMAP_STS_SZ, PROT_READ, 0,
				vi->vi_dfd, SAA_MMAP_STS_OFFSET)) == MAP_FAILED)
		err(errno, "error mapping status buffer");

	s = DefaultScreen(XtDisplay(xi->xi_win));
	if (!XDGAQueryVersion(xi->xi_display, &dummy1, &dummy2) || dummy1 < 2)
                errx(ENOTSUP, 
			"Program require XF86DGA, version 2.0 or greater");
        XF86DGAGetVideoLL(xi->xi_display, s, 
				(int*)&xi->xi_dgainfo->dga_base, 
				&xi->xi_dgainfo->dga_ppl, 
				&dummy1, &dummy2);
	depth = DefaultDepth(xi->xi_display, s);
	/* stopgap till we do fbpp detection */
	if (depth == 24)
		depth = 32;
	xi->xi_dgainfo->dga_bypp = (depth + 1) / 8;
        xi->xi_dgainfo->dga_flags = 0;
	switch (depth) {
                case 32:
                case 24:
                        vi->vi_vfmt = VFMT_RGBa888;
                        break;
                case 16:
                        vi->vi_vfmt = VFMT_RGB565;
                        xi->xi_dgainfo->dga_flags |= VSPEC_FLAG_DITHER;
                        break;
                case 15:
                        vi->vi_vfmt = VFMT_RGBa555;
                        xi->xi_dgainfo->dga_flags |= VSPEC_FLAG_DITHER;
                        break;
		case 8:
                        vi->vi_vfmt = VFMT_Y8;
                        xi->xi_dgainfo->dga_flags |= VSPEC_FLAG_DITHER;
                        break;
                default:
                        errx(ENOTSUP, 
				"error programe require desktop depth "
				"to be greater than or equal to 15,\n"
				"current depth: %d", depth);
                        break;
        }
}


static void
xv_init(struct vinfo *vi, struct xinfo *xi)
{
	struct sigaction sa;
	sigset_t ss;
	Display *d;
	XvAdaptorInfo *adp;
	uint32_t i, j;
	
	sa.sa_flags = 0;
	sigemptyset(&ss);
	sa.sa_mask = ss;
	sa.sa_handler = terminate_notify;
	if (sigaction(SIGINT, &sa, NULL) != 0)
		err(errno, "error setting up terminate signal handler (INT)");
	if (sigaction(SIGHUP, &sa, NULL) != 0)
		err(errno, "error setting up terminate signal handler (HUP)");

 	if (xi->xi_xvinfo->xv_multibuf) {
 		if (xi->xi_xvinfo->xv_nodshm == 0)
 			err(EINVAL, 
 				"Command line error, MULTIBUF (-M) and "
				"DSHM (-X) are mutually exclusive options");
 		sa.sa_handler = xv_multibuf_notify;
 	} else {
 		sa.sa_handler = xv_frame_notify;
 	}
	if (sigaction(SIGUSR1, &sa, NULL) != 0)
		err(errno, "error setting up frame signal handler (USR1)");

	sa.sa_handler = status_notify;
	if (sigaction(SIGUSR2, &sa, NULL) != 0)
		err(errno, "error setting up status signal handler (USR2)");
	if ( (vi->vi_dsts = 
		(struct saa_status*) mmap(0, SAA_MMAP_STS_SZ, PROT_READ, 0,
				vi->vi_dfd, SAA_MMAP_STS_OFFSET)) == MAP_FAILED)
		err(errno, "error mapping status buffer");
	d = xi->xi_display;
	if (XvQueryAdaptors(d,
				XtWindow(xi->xi_win),
				&i, &adp) != Success)
		errx(EIO, "error querying Xv adaptors");
	if (i == 0)
		errx(EIO, "error no Xv adaptors present");
	j = adp[0].base_id + adp[0].num_ports;
	for (i = adp[0].base_id; i < j; ++i)
		if (XvGrabPort(d, i, CurrentTime) == Success)
			goto l1;
	errx(EIO, "error no Xv ports free");
l1:
	xi->xi_xvinfo->xv_port = i;
	XvFreeAdaptorInfo(adp);
	xi->xi_xvinfo->xv_gc = XCreateGC(d, XtWindow(xi->xi_win), 0, NULL);
}


static void
status_notify(int sig)
{
	int dummy;
#if 0
	printf("In status notify AVSTS: 0x%x IREP: 0x%x\n", 
		gvi->vi_dsts->ssts_video_sts,
		gvi->vi_dsts->ssts_irep);
#endif

#if XXX_DISABLED_NEEDS_MORE_WORK
	if ((gvi->vi_dsts->ssts_irep & IREP_RDCAP) ||
	    (gvi->vi_dsts->ssts_video_sts & STS_VIDEO_RDCAP)) {
		usleep(CHANSWITCH_DELAY);
		cntrl_audio_demute();
		if (ioctl(gvi->vi_dfd, VIDEODMACONT, &dummy) != 0)
			warn("DMA pause failed");
	}
#endif
}


static void
xv_multibuf_notify(int sig)
{
	struct saa_status *ssts;
	uint32_t offset;
	
	ssts = gvi->vi_dsts;
	if ((ssts->ssts_video_framectr - gxi->xi_xvinfo->xv_mbuf_ctr) >=
					(ssts->ssts_video_mb_count - 1)) 
	{
		DMSG("Video drop: realigning\n");
		gxi->xi_xvinfo->xv_mbuf_ctr = ssts->ssts_video_framectr - 2;
	}
	offset = gxi->xi_xvinfo->xv_mbuf_ctr % ssts->ssts_video_mb_count;
	offset *= ssts->ssts_video_mbufsz;
	++gxi->xi_xvinfo->xv_mbuf_ctr;

	if (gxi->xi_xvinfo->xv_nodshm) {
#ifdef TVV_USE_MMXEXT
	    special_memcpy_mmxext(gxi->xi_xvinfo->xv_shmaddr, 
					gxi->xi_xvinfo->xv_mmapaddr + offset, 
					gxi->xi_xvinfo->xv_framesz);
#else
	    special_memcpy(gxi->xi_xvinfo->xv_shmaddr, 
					gxi->xi_xvinfo->xv_mmapaddr + offset, 
					gxi->xi_xvinfo->xv_framesz);
#endif
	}

	/* INVARIANT multibuf not allowed in DSHM mode */

	XLockDisplay(gxi->xi_display);
	if (XvShmPutImage(gxi->xi_display, 
			gxi->xi_xvinfo->xv_port, 
			XtWindow(gxi->xi_win), 
			gxi->xi_xvinfo->xv_gc,
			gxi->xi_xvinfo->xv_img,
			0, 0,
			gvi->vi_svs.svs_buf_width,gxi->xi_xvinfo->xv_img_height,
			0, 0,
			gxi->xi_win_width, gxi->xi_win_height,
			False) != Success)
		warn("Xv put image failed");
	XSync(XtDisplay(gxi->xi_win), False);
	if (gvi->vi_dsts->ssts_sig_sts)
		status_notify(sig);
	XUnlockDisplay(gxi->xi_display);
}


static void
xv_frame_notify(int sig)
{
	if (gxi->xi_xvinfo->xv_nodshm) {
#ifdef TVV_USE_MMXEXT
	    special_memcpy_mmxext(gxi->xi_xvinfo->xv_shmaddr, 
					gxi->xi_xvinfo->xv_mmapaddr, 
					gxi->xi_xvinfo->xv_framesz);
#else
	    special_memcpy(gxi->xi_xvinfo->xv_shmaddr, 
					gxi->xi_xvinfo->xv_mmapaddr, 
					gxi->xi_xvinfo->xv_framesz);
#endif
	}

	XLockDisplay(gxi->xi_display);
	if (XvShmPutImage(gxi->xi_display, 
			gxi->xi_xvinfo->xv_port, 
			XtWindow(gxi->xi_win), 
			gxi->xi_xvinfo->xv_gc,
			gxi->xi_xvinfo->xv_img,
			0, 0,
			gvi->vi_svs.svs_buf_width,gxi->xi_xvinfo->xv_img_height,
			0, 0,
			gxi->xi_win_width, gxi->xi_win_height,
			False) != Success)
		warn("Xv put image failed");
	XSync(XtDisplay(gxi->xi_win), False);
	if (gvi->vi_dsts->ssts_sig_sts)
		status_notify(sig);
	XUnlockDisplay(gxi->xi_display);
}


void
terminate_notify(int sig)
{
	stop_video(gvi, gxi);
	if (gxi->xi_xvinfo->xv_shmaddr) {
		if (gxi->xi_xvinfo->xv_img) {  
			XFree(gxi->xi_xvinfo->xv_img);
			gxi->xi_xvinfo->xv_img = 0;
		}
		XShmDetach(gxi->xi_display,&gxi->xi_xvinfo->xv_xshminfo);
		if (shmdt(gxi->xi_xvinfo->xv_shmaddr) == -1)
			warn("Error detaching shared memory");
		gxi->xi_xvinfo->xv_shmaddr = 0;
		shmctl(gxi->xi_xvinfo->xv_shmid, IPC_RMID, 0);
	}

	if (yyin) {
		fseek(yyin, 0, SEEK_SET);
		fprintf(yyin, "%03d\n", *current_channel);
	}
	exit(0);
}


static void
reset_viewport(struct vinfo *vi, struct xinfo *xi)
{
	int i, bpp;
	short x, y, width, height;

	stop_video(vi, xi);
	videoopt_init(vi->vi_dfd, &XV_IMG_WIDTH, &XV_IMG_HEIGHT);
	if (xi->xi_dgainfo->dga_mode) {
		x = xi->xi_win_xpos;
		y = xi->xi_win_ypos;
		width = xi->xi_win_width;
		height = xi->xi_win_height;
		/* XXX use clipping instead, when done */
#if XXX_HANDLED_BY_CLIPPING
		if (x < 0) {
			width += x;
			x = 0;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_NO_HSCALE;
		}
		if (y < 0) {
			height += y;
			y = 0;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_NO_VSCALE;
		}
#endif
		vi->vi_svs.svs_buf_base = (char*)xi->xi_dgainfo->dga_base;
		vi->vi_svs.svs_buf_base += 
		    (y * (xi->xi_dgainfo->dga_ppl * xi->xi_dgainfo->dga_bypp)) +
		    (x * xi->xi_dgainfo->dga_bypp);

		if (width + x >= xi->xi_scr_width) {
			vi->vi_svs.svs_buf_width = xi->xi_scr_width - x;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_NO_HSCALE;
		} else
			vi->vi_svs.svs_buf_width = width;

		if (height + y >= xi->xi_scr_height) {
			vi->vi_svs.svs_buf_height = xi->xi_scr_height - y;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_NO_VSCALE;
		} else
			vi->vi_svs.svs_buf_height = height;

		vi->vi_svs.svs_vfmt = (enum VIDEOFMT)vi->vi_vfmt;
		vi->vi_svs.svs_flags |= 
			xi->xi_dgainfo->dga_flags | VSPEC_FLAG_FRAMELOCK;
		switch (xi->xi_dgainfo->dga_mode) {
		case 'w':
			/* Weave-RT mode 50 ups */
			vi->vi_svs.svs_buf_offset[0][0] = 0;
			vi->vi_svs.svs_buf_offset[1][0] = 
			    xi->xi_dgainfo->dga_ppl * xi->xi_dgainfo->dga_bypp;
			vi->vi_svs.svs_buf_pitch[0] = xi->xi_dgainfo->dga_ppl *
						xi->xi_dgainfo->dga_bypp * 2;
			break;
		case 'b':
			/* Bob mode 50 ups */
			vi->vi_svs.svs_buf_offset[0][0] = 0;
			vi->vi_svs.svs_buf_offset[1][0] = 0;
			vi->vi_svs.svs_buf_pitch[0] = xi->xi_dgainfo->dga_ppl *
						xi->xi_dgainfo->dga_bypp;
			vi->vi_svs.svs_buf_height *= 2;
			break;
		case 'o':
			/* Drop odd field mode 25 ups */
			vi->vi_svs.svs_buf_offset[0][0] = 0;
			vi->vi_svs.svs_buf_offset[1][0] = 0;
			vi->vi_svs.svs_buf_pitch[0] =
			    xi->xi_dgainfo->dga_ppl * xi->xi_dgainfo->dga_bypp;
			vi->vi_svs.svs_buf_height *= 2;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_OVERLAY_DROPODD;
			break;
		case 'e':
			/* Drop even field mode 25 ups */
			vi->vi_svs.svs_buf_offset[0][0] = 0;
			vi->vi_svs.svs_buf_offset[1][0] = 0;
			vi->vi_svs.svs_buf_pitch[0] =
			    xi->xi_dgainfo->dga_ppl * xi->xi_dgainfo->dga_bypp;
			vi->vi_svs.svs_buf_height *= 2;
			vi->vi_svs.svs_flags |= VSPEC_FLAG_OVERLAY_DROPEVEN;
			break;
		}
		vi->vi_svs.svs_flags |= VSPEC_FLAG_STATUS_SIGNAL;
		vi->vi_svs.svs_cap_sigmask = STS_ALL;
	} else {
		/* Xv Mode */
		vi->vi_svs.svs_buf_width = xi->xi_win_width < XV_IMG_WIDTH ?
					xi->xi_win_width : XV_IMG_WIDTH;
		vi->vi_svs.svs_buf_height = xi->xi_win_height < XV_IMG_HEIGHT ?
					xi->xi_win_height : 
					XV_IMG_HEIGHT;
		vi->vi_svs.svs_buf_width = 
			((vi->vi_svs.svs_buf_width + 15) / 16) * 16;
		vi->vi_svs.svs_buf_height = 
			((vi->vi_svs.svs_buf_height + 15) / 16) * 16;
		if (xi->xi_xvinfo->xv_shmaddr) {
			/* XXX Memory Leak?, figure out how to delete XvImage */
			if (xi->xi_xvinfo->xv_img) {  
				XFree(xi->xi_xvinfo->xv_img);
				xi->xi_xvinfo->xv_img = 0;
			}
			XShmDetach(xi->xi_display, 
					&xi->xi_xvinfo->xv_xshminfo);
			if (shmdt(xi->xi_xvinfo->xv_shmaddr) == -1)
				warn("error detaching shared memory");
			xi->xi_xvinfo->xv_shmaddr = 0;
			shmctl(xi->xi_xvinfo->xv_shmid, IPC_RMID, 0);
		}
		i = SHM_R | (SHM_R >> 3) | (SHM_R >> 6);
		i |= SHM_W | (SHM_W >> 3) | (SHM_W >> 6);
		xi->xi_xvinfo->xv_shmid = shmget(TVV_SHMKEY, 
							SAA_VCAP_FRAME_MAX_SZ, 
							i);
		if (xi->xi_xvinfo->xv_shmid != -1) {
			if (shmctl(xi->xi_xvinfo->xv_shmid, IPC_RMID, 0) == -1)
				warnx("error removing stale shared memory "
					"segment, id: 0x%x", 
					xi->xi_xvinfo->xv_shmid);
		}
		i |= IPC_CREAT;
		xi->xi_xvinfo->xv_shmid = shmget(TVV_SHMKEY, 
							SAA_VCAP_FRAME_MAX_SZ,
							i);
		if (xi->xi_xvinfo->xv_shmid == -1)
			err(errno, "error getting shared memory");
		xi->xi_xvinfo->xv_shmaddr = 
			(uint8_t*) shmat(xi->xi_xvinfo->xv_shmid, 0, 0);
		if (xi->xi_xvinfo->xv_shmaddr == (void*)-1)
			err(errno, "error attaching to shared memory");
#if XXX_DISABLED_FOR_HWPIP
		bzero(xi->xi_xvinfo->xv_shmaddr, SAA_VCAP_FRAME_MAX_SZ);
#endif
		xi->xi_xvinfo->xv_xshminfo.shmid = xi->xi_xvinfo->xv_shmid;
		xi->xi_xvinfo->xv_xshminfo.shmaddr = 
				(char*) xi->xi_xvinfo->xv_shmaddr;
		xi->xi_xvinfo->xv_xshminfo.readOnly = False;
		XShmAttach(xi->xi_display, &xi->xi_xvinfo->xv_xshminfo);
		xi->xi_xvinfo->xv_img_height = vi->vi_svs.svs_buf_height;
		/* XXX Weaving not possible with VFMT_PLANAR_YUV420 */
		if (xi->xi_xvinfo->xv_mode == 'b' || 
			xi->xi_xvinfo->xv_mode == 'o' || 
			xi->xi_xvinfo->xv_mode == 'e') {
			xi->xi_xvinfo->xv_img_height /= 2;
			if (xi->xi_xvinfo->xv_fourcc == FOURCC_YV12)
				vi->vi_vfmt = VFMT_PLANAR_YUV420;
		} else {
			if (xi->xi_xvinfo->xv_fourcc == FOURCC_YV12)
				vi->vi_vfmt = VFMT_PLANAR_YUV422;
		}
		xi->xi_xvinfo->xv_img = XvShmCreateImage(xi->xi_display, 
					xi->xi_xvinfo->xv_port,
					xi->xi_xvinfo->xv_fourcc,
					0,
					vi->vi_svs.svs_buf_width, 
					xi->xi_xvinfo->xv_img_height,
					&xi->xi_xvinfo->xv_xshminfo);
		if (!xi->xi_xvinfo->xv_img)
			errx(EIO, "error creating Xv image");
		xi->xi_xvinfo->xv_img->data = (char*)xi->xi_xvinfo->xv_shmaddr;

		vi->vi_svs.svs_cap_nrtasks = 1;
		vi->vi_svs.svs_cap_shmid[0] = xi->xi_xvinfo->xv_shmid;
		vi->vi_svs.svs_vfmt = (enum VIDEOFMT)vi->vi_vfmt;
		vi->vi_svs.svs_cap_swapbytes = vi->vi_vfmt_swapbytes;
		bpp = (vi->vi_vfmt & VFMT_BPP_MASK) >> VFMT_BPP_SHIFT;
		xi->xi_xvinfo->xv_framesz = 
			(xi->xi_xvinfo->xv_img_height * 
					vi->vi_svs.svs_buf_width * bpp) / 8;
		xi->xi_xvinfo->xv_framesz = 
				((xi->xi_xvinfo->xv_framesz + 63) / 64) * 64;
		/*vi->vi_svs.svs_flags == 0, due to decl. as global var. */

		switch (xi->xi_xvinfo->xv_mode) {
		case 'w':	/* Weave */
		case 'r':
			if (xi->xi_xvinfo->xv_fourcc == FOURCC_YUY2 || 
			    xi->xi_xvinfo->xv_fourcc == FOURCC_RGB) {
				vi->vi_svs.svs_buf_offset[0][0] = 0;
				vi->vi_svs.svs_buf_offset[1][0] = 
					(vi->vi_svs.svs_buf_width * bpp) / 8;
				vi->vi_svs.svs_buf_pitch[0] = 
				    (vi->vi_svs.svs_buf_width * bpp * 2) / 8;
			} else {
			    vi->vi_svs.svs_buf_offset[0][0] = 0;
			    vi->vi_svs.svs_buf_offset[1][0] = 
					vi->vi_svs.svs_buf_width;
			    vi->vi_svs.svs_buf_pitch[0] = 
					vi->vi_svs.svs_buf_width * 2;

			    vi->vi_svs.svs_buf_offset[0][2] = 
					vi->vi_svs.svs_buf_width * 
						vi->vi_svs.svs_buf_height;
			    vi->vi_svs.svs_buf_offset[1][2] = 
				vi->vi_svs.svs_buf_offset[0][2] + 
				(vi->vi_svs.svs_buf_width * 
					vi->vi_svs.svs_buf_height / 2);
			    vi->vi_svs.svs_buf_pitch[2] = 
					vi->vi_svs.svs_buf_width / 2;

			    vi->vi_svs.svs_buf_offset[0][1] = 
				vi->vi_svs.svs_buf_width * 
				vi->vi_svs.svs_buf_height +
			 	(vi->vi_svs.svs_buf_width * 
					vi->vi_svs.svs_buf_height / 4);
			    vi->vi_svs.svs_buf_offset[1][1] = 
				vi->vi_svs.svs_buf_offset[0][1] + 
				(vi->vi_svs.svs_buf_width * 
					vi->vi_svs.svs_buf_height / 2);
			    vi->vi_svs.svs_buf_pitch[1] = 
					vi->vi_svs.svs_buf_width / 2;
			}
			if (xi->xi_xvinfo->xv_mode == 'r')	/* Weave-RT */
				vi->vi_svs.svs_flags |= 
					VSPEC_FLAG_CAPTURE_SIGFLD;
			break;
		case 'b':
			if (xi->xi_xvinfo->xv_fourcc == FOURCC_YUY2 || 
			    xi->xi_xvinfo->xv_fourcc == FOURCC_RGB) {
				vi->vi_svs.svs_buf_offset[0][0] = 0;
				vi->vi_svs.svs_buf_offset[1][0] = 0;
				vi->vi_svs.svs_buf_pitch[0] = 
					(vi->vi_svs.svs_buf_width * bpp) / 8;
			} else {
			    vi->vi_svs.svs_buf_offset[0][0] = 0;
			    vi->vi_svs.svs_buf_offset[1][0] = 0;
			    vi->vi_svs.svs_buf_pitch[0] = 
					vi->vi_svs.svs_buf_width;

			    vi->vi_svs.svs_buf_offset[0][2] = 
				(vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height);
			    vi->vi_svs.svs_buf_offset[1][2] = 
				vi->vi_svs.svs_buf_offset[0][2];
			    vi->vi_svs.svs_buf_pitch[2] = 
				vi->vi_svs.svs_buf_width / 2;

			    vi->vi_svs.svs_buf_offset[0][1] = 
				(vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height) +
			 	((vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height) / 4);
			    vi->vi_svs.svs_buf_offset[1][1] = 
					vi->vi_svs.svs_buf_offset[0][1];
			    vi->vi_svs.svs_buf_pitch[1] = 
					vi->vi_svs.svs_buf_width / 2;
			}
			vi->vi_svs.svs_flags |= VSPEC_FLAG_CAPTURE_SIGFLD;
			break;
		case 'o':
		case 'e':
			if (xi->xi_xvinfo->xv_fourcc == FOURCC_YUY2 || 
			    xi->xi_xvinfo->xv_fourcc == FOURCC_RGB) {
				vi->vi_svs.svs_buf_offset[0][0] = 0;
				vi->vi_svs.svs_buf_pitch[0] = 
					(vi->vi_svs.svs_buf_width * bpp) / 8;
				vi->vi_svs.svs_buf_offset[1][0] = 
					(vi->vi_svs.svs_buf_pitch[0] * 
						vi->vi_svs.svs_buf_height) / 2;
				if (xi->xi_xvinfo->xv_mode == 'e') {
					vi->vi_svs.svs_buf_offset[0][0] = 
						vi->vi_svs.svs_buf_offset[0][1];
					vi->vi_svs.svs_buf_offset[0][1] = 0;
				}
			} else {
			    i = (vi->vi_svs.svs_buf_width * 
				xi->xi_xvinfo->xv_img_height * 3) / 2;
			    vi->vi_svs.svs_buf_offset[0][0] = 0;
			    vi->vi_svs.svs_buf_offset[1][0] = i;
			    vi->vi_svs.svs_buf_pitch[0] = 
				vi->vi_svs.svs_buf_width;

			    vi->vi_svs.svs_buf_offset[0][2] = 
				(vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height);
			    vi->vi_svs.svs_buf_offset[1][2] = 
				vi->vi_svs.svs_buf_offset[0][2] + i;
			    vi->vi_svs.svs_buf_pitch[2] = 
				vi->vi_svs.svs_buf_width / 2;

			    vi->vi_svs.svs_buf_offset[0][1] = 
				(vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height) +
			 	((vi->vi_svs.svs_buf_width * 
					xi->xi_xvinfo->xv_img_height) / 4);
			    vi->vi_svs.svs_buf_offset[1][1] = 
				vi->vi_svs.svs_buf_offset[0][1] + i;
			    vi->vi_svs.svs_buf_pitch[1] = 
					vi->vi_svs.svs_buf_width / 2;
			    if (xi->xi_xvinfo->xv_mode == 'e') {
				i = vi->vi_svs.svs_buf_offset[0][0];
				vi->vi_svs.svs_buf_offset[0][0] = 
					vi->vi_svs.svs_buf_offset[1][0];
				vi->vi_svs.svs_buf_offset[1][0] = i;

				i = vi->vi_svs.svs_buf_offset[0][1];
				vi->vi_svs.svs_buf_offset[0][1] = 
					vi->vi_svs.svs_buf_offset[1][1];
				vi->vi_svs.svs_buf_offset[1][1] = i;

				i = vi->vi_svs.svs_buf_offset[0][2];
				vi->vi_svs.svs_buf_offset[0][2] = 
					vi->vi_svs.svs_buf_offset[1][2];
				vi->vi_svs.svs_buf_offset[1][2] = i;
			    }
			}
			break;
		}
		vi->vi_svs.svs_flags |= VSPEC_FLAG_CAPTURE_SIGNAL |
					VSPEC_FLAG_STATUS_SIGNAL |
					VSPEC_FLAG_FRAMELOCK;
		vi->vi_svs.svs_cap_sigmask = STS_ALL;
		if (xi->xi_xvinfo->xv_nodshm) {
			  vi->vi_svs.svs_cap_bufsz = SAA_VCAP_FRAME_MAX_SZ;
		} else
			vi->vi_svs.svs_flags |= VSPEC_FLAG_CAPTURE_SHM;
		if (xi->xi_xvinfo->xv_multibuf) {
			vi->vi_svs.svs_flags |= VSPEC_FLAG_CAPTURE_MULTIBUF;
		}
	}
}


void
stop_video(struct vinfo *vi, struct xinfo *xi)
{
	int dummy;
	if (xi->xi_dgainfo->dga_mode) {
		if (ioctl(vi->vi_dfd, OVERLAYOFF, &dummy) != 0)
			warn("DGA stop_video failed");
	} else {
/* Add to atexit dnit code */
#if XXX_OBSOLETE
/* XXX Not required due to saa device buffer allocation strategy ? */
		if (xi->xi_xvinfo->xv_nodshm) {
			if (xi->xi_xvinfo->xv_mmaplen) {
			    	if (munmap(xi->xi_xvinfo->xv_mmapaddr, 
					SAA_VCAP_FRAME_MAX_SZ)!= 0)
					warn("munmap failure, mem leak!\n");
			}
		}
#endif
		if (ioctl(vi->vi_dfd, CAPTUREOFF, &dummy) != 0)
			warn("Xv stop_video failed");
	}
	DMSG("Stopping Video\n");
	vi->vi_flag_on = 0;
}


void
start_video(struct vinfo *vi, struct xinfo *xi)
{
	DMSG("Starting Video\n");
	if (xi->xi_dgainfo->dga_mode) {
		generate_saa_cliplist(xi, &vi->vi_svs.svs_cliplist);
		if (ioctl(vi->vi_dfd, OVERLAYON, &vi->vi_svs) != 0)
			warn("DGA start video failed");
		else
			vi->vi_flag_on = 1;
	} else {
		if (ioctl(vi->vi_dfd, CAPTUREON, &vi->vi_svs) != 0)
			warn("Device start video failed");
		else {
			vi->vi_flag_on = 1;
			if (xi->xi_xvinfo->xv_nodshm) {
				if (xi->xi_xvinfo->xv_mmapaddr == 0)
					xi->xi_xvinfo->xv_mmapaddr = (uint8_t*)
						mmap(0, 
							SAA_VCAP_FRAME_MAX_SZ,
							PROT_READ, 0,
							vi->vi_dfd, 
							SAA_MMAP_T0_OFFSET); 
				if (xi->xi_xvinfo->xv_mmapaddr == MAP_FAILED)
				    err(errno, 
					"Error mapping device frame buffer");
			}
		}
	}
}


void
resize_video_win(int w, int h)
{
	XV_IMG_WIDTH = w;
	XV_IMG_HEIGHT = h;
	XLockDisplay(gxi->xi_display);
	XUnmapWindow(gxi->xi_display, XtWindow(gxi->xi_win));
	XtVaSetValues(gxi->xi_win, 
			XtNwidth, w,
			XtNheight, h,
			NULL);
	XMapWindow(gxi->xi_display, XtWindow(gxi->xi_win));
	XUnlockDisplay(gxi->xi_display);
}


static void
generate_saa_cliplist(struct xinfo *xi, struct saa_video_cliplist *svcl)
{
	uint32_t i;
#if XXX_TESTCASE
	/* XXX Figure out why this combination does not
	 * work, I suspect hw deficency
	 */
	svcl->svcl_count = 2;
	svcl->svcl_clip[0].x = 0;
	svcl->svcl_clip[0].y = 0;
	svcl->svcl_clip[0].w = 287;
	svcl->svcl_clip[0].h = 576;

	svcl->svcl_clip[1].x = 0;
	svcl->svcl_clip[1].y = 0;
	svcl->svcl_clip[1].w = 720;
	svcl->svcl_clip[1].h = 102;

#else
	svcl->svcl_count = xi->xi_set_clist_count;
	for (i = 0; i < xi->xi_set_clist_count; ++i) {
		svcl->svcl_clip[i].x = xi->xi_set_clist[i].ci_a.x;
		svcl->svcl_clip[i].y = xi->xi_set_clist[i].ci_a.y;
		svcl->svcl_clip[i].w = xi->xi_set_clist[i].ci_a.w;
		svcl->svcl_clip[i].h = xi->xi_set_clist[i].ci_a.h;
	}
#endif
}


static int
xgetwinattr_errorhandler(Display *d, XErrorEvent *e) 
{
	return TRUE;
}
