#include <sys/param.h>
#include <sys/ioctl.h>
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <saa713x_ioctl.h>

#include "videoopt.h"

#define NR_MODES	9

static enum MODE VSRC_TRANSLATE[NR_MODES] = {
	MODE_CVBS_1,
	MODE_CVBS_2,
	MODE_CVBS_3,
	MODE_CVBS_4,
	MODE_CVBS_5,
	MODE_SVIDEOAGC_1,
	MODE_SVIDEOAGC_2,
	MODE_SVIDEOFIXGC_1,
	MODE_SVIDEOFIXGC_2,
};

static struct saa_video_std video_std_pal = {
	.std_first_linenum  		= 1,
	.std_video_xinstart  		= 0,
	.std_video_xinstop 		= 719,
	.std_video_yinstart 		= 21,
	.std_video_yinstop 		= 21 + 287,
	.std_vbi_xinstart  		= 0,
	.std_vbi_xinstop 		= 0,
	.std_vbi_yinstart		= 0,
	.std_vbi_yinstop		= 0,
	.std_brightness 		= 0x80,
	.std_contrast 			= 0x44,
	.std_saturation 		= 0x40,
	.std_vbi_raw_src 		= VBIRAWSRC_YUV
};


static struct saa_video_std video_std_ntsc = {	/* PAL B,G defaults */ 
	.std_first_linenum  		= 1,
	.std_video_xinstart  		= 0,
	.std_video_xinstop 		= 719,
	.std_video_yinstart 		= 16,
	.std_video_yinstop 		= 16 + 239,
	.std_vbi_xinstart  		= 0,
	.std_vbi_xinstop 		= 0,
	.std_vbi_yinstart		= 0,
	.std_vbi_yinstop		= 0,
	.std_brightness 		= 0x80,
	.std_contrast 			= 0x44,
	.std_saturation 		= 0x40,
	.std_vbi_raw_src 		= VBIRAWSRC_YUV
};


static struct saa_video_opt video_opt_cvbs = { 
	.svo_flag_wpoff			= 0,
	.svo_fuse 			= FUSE_GAON_AAOFF,
	.svo_input_mode 		= MODE_SVIDEOAGC_1,
	.svo_flag_dtv			= 0,
	.svo_flag_copyprotsrc		= 0,
	.svo_flag_cpoff  		= 0,
	.svo_flag_holdg			= 0,
	.svo_flag_fixgain  		= 0,
	.svo_adc1_gain  		= 0,
	.svo_adc2_gain  		= 0,
	.svo_feild_rate  		= FRATE_60,
	.svo_htc_mode  			= HTCMODE_FAST,
	.svo_flag_nonpll_mode  		= 0,
	.svo_vno_mode  			= VNOMODE_NORM,
	.svo_flag_byps 			= 1,
	.svo_flag_ycomb 		= 1,
	.svo_flag_lubw 			= 1,
	.svo_lufi 			= 8,
	.svo_brightness 		= 0x80,
	.svo_contrast 			= 0x44,
	.svo_saturation 		= 0x40,
	.svo_hue 			= 0x00,
	.svo_flag_no_acgc 		= 0,
	.svo_chroma_gain 		= 0,
	.svo_flag_cdto 			= 0,
	.svo_cstd 			= CSTD_NTSC,
	.svo_flag_dcvf 			= 0,
	.svo_flag_fctc 			= 0,
	.svo_flag_ccomb 		= 1,
	.svo_offv 			= 0,
	.svo_offu 			= 0,
	.svo_flag_chbw 			= 1,
	.svo_lcbw 			= 7,
	.svo_ydel			= 0,
	.svo_hdel			= 0,
	.svo_flag_forcecolor		= 0,
	.svo_apck			= 1,
	.svo_aosl			= 0,
	.svo_flag_uptcv			= 1,
	.svo_flag_cm99			= 0,
	.svo_laty			= 3,
	.svo_adetect_std 		= ADETECT_DISABLE,
};


static struct saa_video_opt video_opt_tuner = { 
	.svo_flag_wpoff			= 0,

	.svo_fuse 			= FUSE_GAON_AAOFF,
	.svo_input_mode 		= MODE_CVBS_2,

	.svo_flag_dtv			= 0,
	.svo_flag_copyprotsrc		= 0,
	.svo_flag_cpoff  		= 0,
	.svo_flag_holdg			= 0,
	.svo_flag_fixgain  		= 0,

	.svo_adc1_gain  		= 0,
	.svo_adc2_gain  		= 0,

	.svo_feild_rate  		= FRATE_50,
	.svo_htc_mode  			= HTCMODE_FAST,
	.svo_flag_nonpll_mode  		= 0,
	.svo_vno_mode  			= VNOMODE_NORM,

	.svo_flag_byps 			= 0,
	.svo_flag_ycomb 		= 1,
	.svo_flag_lubw 			= 1,
	.svo_lufi 			= 8,

	.svo_brightness 		= 0x80,
	.svo_contrast 			= 0x44,
	.svo_saturation 		= 0x40,
	.svo_hue 			= 0x00,

	.svo_flag_no_acgc 		= 0,
	.svo_chroma_gain 		= 0,

	.svo_flag_cdto 			= 0,
	.svo_cstd 			= CSTD_PAL,
	.svo_flag_dcvf 			= 0,
	.svo_flag_fctc 			= 0,
	.svo_flag_ccomb 		= 1,

	.svo_offv 			= 0,
	.svo_offu 			= 0,
	.svo_flag_chbw 			= 1,
	.svo_lcbw 			= 7,

	.svo_ydel			= 0,
	.svo_hdel			= 0,
	.svo_flag_forcecolor		= 0,

	.svo_apck			= 1,
	.svo_aosl			= 0,
	.svo_flag_uptcv			= 1,
	.svo_flag_cm99			= 0,

	.svo_laty			= 3,

	.svo_adetect_std 		= ADETECT_DISABLE,
};


static struct saa_video_opt vopt_current;
static struct saa_video_std vstd_current;
static int dfd;

void
videoopt_init(int d, int *img_width, int *img_height)
{
	dfd = d;

	if (ioctl(dfd, GETVIDEOSTD, &vstd_current) != 0)
		errx(errno, "error getting video standard");

	if (ioctl(dfd, GETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error getting video options");

	*img_width = vstd_current.std_video_xinstop - 
			vstd_current.std_video_xinstart + 1;
	*img_height = (vstd_current.std_video_yinstop - 
			vstd_current.std_video_yinstart + 1) * 2;
}


void
videoopt_set_std(enum VSTD vstd, int *img_width, int *img_height)
{
	if (vstd == VSTD_PAL) {
		vstd_current = video_std_pal;
	} else if (vstd == VSTD_NTSC) {
		vstd_current = video_std_ntsc;
	} else
		errx(EDOOFUS, "invalid video standard %d", vstd);

	if (ioctl(dfd, SETVIDEOSTD, &vstd_current) != 0)
		errx(errno, "error setting video standard");

	*img_width = vstd_current.std_video_xinstop - 
			vstd_current.std_video_xinstart + 1;
	*img_height = (vstd_current.std_video_yinstop - 
			vstd_current.std_video_yinstart + 1) * 2;
}

int
videoopt_get_brightness()
{
	return vopt_current.svo_brightness;
}

void
videoopt_set_brightness(int b)
{
	vopt_current.svo_brightness = b;
	if (ioctl(dfd, SETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error setting video options");
}


int
videoopt_get_contrast()
{
	return vopt_current.svo_contrast;
}

void
videoopt_set_contrast(int b)
{
	vopt_current.svo_contrast = b;
	if (ioctl(dfd, SETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error setting video options");
}


int
videoopt_get_saturation()
{
	return vopt_current.svo_saturation;
}

void
videoopt_set_saturation(int b)
{
	vopt_current.svo_saturation = b;
	if (ioctl(dfd, SETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error setting video options");
}

int
videoopt_get_hue()
{
	return vopt_current.svo_hue;
}

void
videoopt_set_hue(int b)
{
	vopt_current.svo_hue = b;
	if (ioctl(dfd, SETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error setting video options");
}

/* Before switching save current brightness, etc.. */
void
videoopt_set_src(enum VSRC vsrc)
{
	static enum MODE last_mode = (enum MODE) -1;
	enum MODE m;

	if (vsrc >= NR_MODES)
		errx(EDOOFUS, "critical programing error," 
				" set_src invalid VSRC");

	m = VSRC_TRANSLATE[vsrc];

	if (last_mode == m)
		return;

	if (m >= MODE_SVIDEOFIXGC_1) {
		if (last_mode != (enum MODE)-1)
			video_opt_tuner = vopt_current;
		vopt_current = video_opt_cvbs;
	} else {
		if (last_mode != -1)
			video_opt_cvbs = vopt_current;
		vopt_current = video_opt_tuner;
	}
	vopt_current.svo_input_mode = m;
	last_mode = m;

	if (ioctl(dfd, SETVIDEOOPT, &vopt_current) != 0)
		errx(errno, "error setting video options");
}


enum VSRC 
videoopt_get_src()
{
	int i;

	for (i = 0; i < NR_MODES; ++i) {
		if (vopt_current.svo_input_mode == VSRC_TRANSLATE[i])
			return (enum VSRC) i;
	}
	errx(EDOOFUS, "critical programing error, NR_MODES search failed");
	return (enum VSRC) 0;
}


enum
VSTD videoopt_get_std()
{
	if (vstd_current.std_video_yinstop - 
		vstd_current.std_video_yinstart < 250)
		return VSTD_NTSC;
	else
		return VSTD_PAL;
}
